#ifndef __PT_BLEP_H
#define __PT_BLEP_H

// thanks to aciddose/ad/adejr for the blep/cutoff/filter stuff!

// information on blep variables
//
// ZC = zero crossings, the number of ripples in the impulse
// OS = oversampling, how many samples per zero crossing are taken
// SP = step size per output sample, used to lower the cutoff (play the impulse slower)
// NS = number of samples of impulse to insert 
// RNS = the lowest power of two greater than NS, minus one (used to wrap output buffer)
//
// ZC and OS are here only for reference, they depend upon the data in the table and can't be changed.
// SP, the step size can be any number lower or equal to OS, as long as the result NS remains an integer.
// for example, if ZC=8,OS=5, you can set SP=1, the result is NS=40, and RNS must then be 63.
// the result of that is the filter cutoff is set at nyquist * (SP/OS), in this case nyquist/5.

#define ZC 8
#define OS 5
#define SP 5
#define NS (ZC * OS / SP)
#define RNS 7 // RNS = (2^ > NS) - 1

typedef struct blep_data
{
    signed int index;
    float lastValue;
    signed int samplesLeft;
    float buffer[RNS + 1];
} BLEP;

void blepAdd(BLEP *b, float offset, float amplitude);
float blepRun(BLEP *b);

#endif
