#include <stdio.h>
#include "pt_header.h"
#include "pt_helpers.h"
#include "pt_visuals.h"
#include "pt_palette.h"
#include "pt_diskop.h"
#include "pt_edit.h"
#include "pt_sampler.h"
#include "pt_audio.h"
#include "pt_buttons.h"

// taken from the ptplay project

static const pt_button_s bAllScreens[4] =
{
    {  0,  0,  0,  0, PTB_QUIT},
    {  0,100,320,110, PTB_SONGNAME},
    {  0,111,286,120, PTB_SAMPLENAME},
    {287,110,320,120, PTB_LOADSAMPLE}
};

static const pt_button_s bTopScreen[45] =
{
    {120,  0,181, 10, PTB_PLAY},
    {120, 11,181, 21, PTB_PATTERN},
    {120, 21,181, 32, PTB_EDIT},
    {120, 33,181, 43, PTB_RECORD},
    {182,  0,243, 10, PTB_STOP},
    {182, 11,243, 21, PTB_CLEAR},
    {182, 21,243, 32, PTB_EDITOP},
    {182, 33,243, 43, PTB_DISKOP},
    {244,  0,305, 10, PTB_PLST},
    {244, 11,305, 21, PTB_PSETED},
    {244, 21,305, 32, PTB_SETUP},
    {244, 33,305, 43, PTB_SAMPLER},
    {306,  0,319, 10, PTB_CHAN1},
    {306, 11,319, 21, PTB_CHAN2},
    {306, 21,319, 32, PTB_CHAN3},
    {306, 33,319, 43, PTB_CHAN4},
    { 70,  0, 97, 10, PTB_POSS},
    { 98,  0,108, 10, PTB_POSU},
    {109,  0,119, 10, PTB_POSD},
    { 70, 11, 97, 21, PTB_PATTERNS},
    { 98, 11,108, 21, PTB_PATTERNU},
    {109, 11,119, 21, PTB_PATTERND},
    { 70, 22, 97, 32, PTB_PLENGTHS},
    { 98, 22,108, 32, PTB_PLENGTHU},
    {109, 22,119, 32, PTB_PLENGTHD},
    { 78, 33, 97, 43, PTB_FTUNES},
    { 98, 33,108, 43, PTB_FTUNEU},
    {109, 33,119, 43, PTB_FTUNED},
    { 78, 44, 97, 54, PTB_SAMPLES},
    { 98, 44,108, 54, PTB_SAMPLEU},
    {109, 44,119, 54, PTB_SAMPLED},
    { 78, 55, 97, 65, PTB_SVOLUMES},
    { 98, 55,108, 65, PTB_SVOLUMEU},
    {109, 55,119, 65, PTB_SVOLUMED},
    { 54, 66, 97, 76, PTB_SLENGTHS},
    { 98, 66,108, 76, PTB_SLENGTHU},
    {109, 66,119, 76, PTB_SLENGTHD},
    { 54, 77, 97, 87, PTB_SREPEATS},
    { 98, 77,108, 87, PTB_SREPEATU},
    {109, 77,119, 87, PTB_SREPEATD},
    { 54, 88, 97, 98, PTB_SREPLENS},
    { 98, 88,108, 98, PTB_SREPLENU},
    {109, 88,119, 98, PTB_SREPLEND},
    {  1,  1, 69, 10, PTB_POSED},
    {120, 44,320, 99, PTB_VISUALS}
};

static const pt_button_s bBottomScreen[2] =
{
    { 26,122, 44,136, PTB_TEMPOU},
    { 45,122, 59,136, PTB_TEMPOD}
};

static const pt_button_s bDiskOp[26] =
{
    {  1,  1, 82, 10, PTB_DO_FORMATDISK},
    { 83,  0,145, 10, PTB_DO_PACKMOD},
    {146,  0,155, 10, PTB_DO_MODARROW},
    {156,  0,237, 10, PTB_DO_LOADMODULE},
    {238,  0,319, 10, PTB_DO_SAVEMODULE},
    {  0, 11, 82, 20, PTB_DO_RENAMEFILE},
    { 83, 11,145, 20, PTB_DO_PACKSONG},
    {146, 11,155, 20, PTB_DO_SONGARROW},
    {156, 11,237, 20, PTB_DO_LOADSONG},
    {238, 11,319, 20, PTB_DO_SAVESONG},
    {  0, 22, 82, 33, PTB_DO_DELETEFILE},
    { 83, 22,145, 33, PTB_DO_SAMPLEFORMAT},
    {146, 22,155, 33, PTB_DO_SAMPLEARROW},
    {156, 22,237, 33, PTB_DO_LOADSAMPLE},
    {238, 22,319, 33, PTB_DO_SAVESAMPLE},
    {  0, 33, 10, 43, PTB_DO_TOGGLEPATH},
    { 11, 33,176, 42, PTB_DO_DATAPATH},
    {177, 33,186, 42, PTB_DO_SWAPPAGE},
    {187, 33,216, 42, PTB_DO_PARENT},
    {217, 33,307, 42, PTB_DO_FREEMEM},
    {308, 33,319, 42, PTB_DO_SCROLLTOP},
    {308, 43,319, 50, PTB_DO_SCROLLUP},
    {308, 51,319, 81, PTB_DO_EXIT},
    {308, 82,319, 89, PTB_DO_SCROLLDOWN},
    {308, 90,319, 99, PTB_DO_SCROLLBOT},
    {  2, 44,304, 93, PTB_DO_FILEAREA}
};

static const pt_button_s bQuit[2] =
{
    {171, 71,196, 81, PTB_SUREY},
    {234, 71,252, 81, PTB_SUREN}
};

static const pt_button_s bClear[4] =
{
    {166, 57,198, 67, PTB_CLEARSONG},
    {204, 57,257, 67, PTB_CLEARSAMPLES},
    {166, 73,198, 83, PTB_CLEARALL},
    {204, 73,257, 83, PTB_CLEARCANCEL}
};

static const pt_button_s bSampler[25] =
{ 
    {  6,124, 25,134, PTB_SA_EXIT},
    { 32,212, 95,220, PTB_SA_PLAYWAVE},
    { 96,212,175,220, PTB_SA_SHOWRANGE},
    {176,212,245,220, PTB_SA_ZOOMOUT},
    {  0,223, 30,242, PTB_SA_STOP},
    { 32,223, 95,231, PTB_SA_PLAYDISPLAYED},
    { 96,223,175,231, PTB_SA_SHOWALL},
    {176,223,245,231, PTB_SA_RANGEALL},
    {246,223,319,231, PTB_SA_LOOP},
    { 32,234, 94,242, PTB_SA_PLAYRANGE},
    { 96,234,114,242, PTB_SA_RANGEBEG},
    {116,234,135,242, PTB_SA_RANGEEND},
    {136,234,174,242, PTB_SA_SWAPBUF},
    {176,234,245,242, PTB_SA_SAMPLE},
    {246,234,319,242, PTB_SA_SAMPLENOTE},
    {  0,245, 31,256, PTB_SA_CUT},
    { 32,245, 63,256, PTB_SA_COPY},
    { 64,245, 95,256, PTB_SA_PASTE},
    { 96,245,135,256, PTB_SA_VOLUME},
    {136,245,175,256, PTB_SA_TUNETONE},
    {176,245,245,256, PTB_SA_RESAMPLE},
    {246,245,319,256, PTB_SA_RESAMPLENOTE},
    {  0,138,319,201, PTB_SA_SAMPLEAREA},
    {  3,205,316,210, PTB_SA_ZOOMBARAREA}
};

int checkGUIButtons(int mouseX, int mouseY)
{
    int i;

    if (editor.ui.exitScreenShown == true)
    {
        for (i = 0; i < 2; ++i) // EXIT BUTTONS
        {
            if ((mouseX >= bQuit[i].x1) && (mouseX <= bQuit[i].x2)
             && (mouseY >= bQuit[i].y1) && (mouseY <= bQuit[i].y2))
            {
                 return bQuit[i].b;
            }
        }
    }
    else if (editor.ui.clearScreenShown == true)
    {
        for (i = 0; i < 4; ++i) // CLEAR BUTTONS
        {
            if ((mouseX >= bClear[i].x1) && (mouseX <= bClear[i].x2)
             && (mouseY >= bClear[i].y1) && (mouseY <= bClear[i].y2))
            {
                 return bClear[i].b;
            }
        }
    }
    else
    {
        // ALL SCREENS BUTTONS
        for (i = 0; i < 4; ++i)
        {
            if ((mouseX >= bAllScreens[i].x1) && (mouseX <= bAllScreens[i].x2)
                && (mouseY >= bAllScreens[i].y1) && (mouseY <= bAllScreens[i].y2))
            {
                return bAllScreens[i].b;
            }
        }

        if (editor.ui.diskOpScreenShown == true)
        {
            if (editor.ui.samplerScreenShown == true)
            {
                for (i = 0; i < 24; ++i) // SAMPLER BUTTONS
                {
                    if ((mouseX >= bSampler[i].x1) && (mouseX <= bSampler[i].x2)
                     && (mouseY >= bSampler[i].y1) && (mouseY <= bSampler[i].y2))
                    {
                        return bSampler[i].b;
                    }
                }
            }
            else
            {
                // BOTTOM SCREEN
                for (i = 0; i < 2; ++i)
                {
                    if ((mouseX >= bBottomScreen[i].x1) && (mouseX <= bBottomScreen[i].x2)
                     && (mouseY >= bBottomScreen[i].y1) && (mouseY <= bBottomScreen[i].y2))
                    {
                        return bBottomScreen[i].b;
                    }
                }
            }

            for (i = 0; i < 27; ++i) // DISKOP BUTTONS
            {
                if ((mouseX >= bDiskOp[i].x1) && (mouseX <= bDiskOp[i].x2)
                 && (mouseY >= bDiskOp[i].y1) && (mouseY <= bDiskOp[i].y2))
                {
                     return bDiskOp[i].b;
                }
            }
        }
        else
        {
            // TOP SCREEN
            for (i = 0; i < 45; ++i)
            {
                if ((mouseX >= bTopScreen[i].x1) && (mouseX <= bTopScreen[i].x2)
                    && (mouseY >= bTopScreen[i].y1) && (mouseY <= bTopScreen[i].y2))
                {
                    return bTopScreen[i].b;
                }
            }
        }
            
        if (editor.ui.samplerScreenShown == true)
        {
            if (editor.ui.diskOpScreenShown == true)
            {
                for (i = 0; i < 27; ++i) // DISKOP BUTTONS
                {
                    if ((mouseX >= bDiskOp[i].x1) && (mouseX <= bDiskOp[i].x2)
                     && (mouseY >= bDiskOp[i].y1) && (mouseY <= bDiskOp[i].y2))
                    {
                         return bDiskOp[i].b;
                    }
                }
            }
            else
            {
                // TOP SCREEN
                for (i = 0; i < 45; ++i)
                {
                    if ((mouseX >= bTopScreen[i].x1) && (mouseX <= bTopScreen[i].x2)
                     && (mouseY >= bTopScreen[i].y1) && (mouseY <= bTopScreen[i].y2))
                    {
                        return bTopScreen[i].b;
                    }
                }
            }

            for (i = 0; i < 71; ++i) // SAMPLER BUTTONS
            {
                if ((mouseX >= bSampler[i].x1) && (mouseX <= bSampler[i].x2)
                 && (mouseY >= bSampler[i].y1) && (mouseY <= bSampler[i].y2))
                {
                    if ((bSampler[i].b < 46) && (editor.ui.diskOpScreenShown == true))
                    {
                        return -1;
                    }

                    return bSampler[i].b;
                }
            }
        }
        else
        {
            // BOTTOM SCREEN
            for (i = 0; i < 2; ++i)
            {
                if ((mouseX >= bBottomScreen[i].x1) && (mouseX <= bBottomScreen[i].x2)
                    && (mouseY >= bBottomScreen[i].y1) && (mouseY <= bBottomScreen[i].y2))
                {
                    return bBottomScreen[i].b;
                }
            }
        }
    }

    return -1;
}

void movePatCurPrevCh(void);
void movePatCurNextCh(void);
void movePatCurRight(void);
void movePatCurLeft(void);

void keyUpHandler(SDLKey keyEntry)
{
    switch (keyEntry)
    {
        case SDLK_UP:
        case SDLK_DOWN:
        case SDLK_LEFT:
        case SDLK_RIGHT:
        {
            input.keyb.cursorKeyPressed = false;
        }
        break;

        case SDLK_LCTRL:
        {
            input.keyb.leftControlKeyDown = false;
        }
        break;

        case SDLK_LSHIFT:
        {
            input.keyb.leftShiftKeyDown = false;
        }
        break;

        case SDLK_LALT:
        {
            input.keyb.leftAltKeyDown = false;
        }
        break;

        case SDLK_KP_ENTER:
        {
            input.keyb.keypadEnterKeyDown = false;
        }
        break;

        default:
        break;
    }
}

void keyDownHandler(SDLKey keyEntry, const SDL_Surface *screen, MODULE *modEntry)
{
    int sampleOffset;

    if (input.keyb.keypadEnterKeyDown == true)
    {
        sampleOffset = 0x10;
    }
    else
    {
        sampleOffset = 0;
    }

    switch (keyEntry)
    {
        case SDLK_ESCAPE:
        {
            /* TOP LAYER DIALOGS */
            if (editor.ui.exitScreenShown == true)
            {
                editor.ui.exitScreenShown = false;
            }
            else if (editor.ui.clearScreenShown == true)
            {
                editor.ui.clearScreenShown = false;
            }
            /* END OF TOP LAYER DIALOGS */

            else if (editor.ui.diskOpScreenShown == true)
            {
                editor.ui.diskOpScreenShown = false;
            }
            else if (editor.ui.samplerScreenShown == true)
            {
                editor.ui.samplerScreenShown = false;
            }
            else
            {
                editor.ui.exitScreenShown = true;

                pointerSetMode(POINTER_MODE_MSG1, false);
                setStatusMessage("REALLY QUIT ?", false);

                return;
            }

            pointerSetPreviousMode();
            setPrevStatusMessage();
        }
        break;

        case SDLK_RETURN:
        {
            if (editor.ui.exitScreenShown == true)
            {
                editor.ui.throwExit = true;

                break;
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_y:
        {
            if (editor.ui.exitScreenShown == true)
            {
                editor.ui.throwExit = true;
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_n:
        {
            if (editor.ui.exitScreenShown == true)
            {
                editor.ui.exitScreenShown = false;

                pointerSetPreviousMode();
                setPrevStatusMessage();
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        // toggle between IDLE and EDIT (IDLE if PLAY)
        case SDLK_SPACE:
        {
            if (editor.currentMode == MODE_PLAY)
            {
                modStop();

                editor.currentMode = MODE_IDLE;
                pointerSetMode(POINTER_MODE_IDLE, true);
                setStatusMessage("ALL RIGHT", true);
            }
            else if ((editor.currentMode == MODE_EDIT) || (editor.currentMode == MODE_RECORD))
            {
                modStop();

                editor.currentMode = MODE_IDLE;
                pointerSetMode(POINTER_MODE_IDLE, true);
                setStatusMessage("ALL RIGHT", true);
            }
            else
            {
                modStop();

                editor.currentMode = MODE_EDIT;
                pointerSetMode(POINTER_MODE_EDIT, true);
                setStatusMessage("ALL RIGHT", true);
            }
        }
        break;

        case SDLK_F1:
        {
            editor.keyOctave = OCTAVE_LOW;
        }
        break;

        case SDLK_F2:
        {
            editor.keyOctave = OCTAVE_HIGH;
        }
        break;

        case SDLK_F6:
        {
            if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
            {
                modSetPos(-1, 0);
            }
        }
        break;

        case SDLK_F7:
        {
            if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
            {
                modSetPos(-1, 16);
            }
        }
        break;

        case SDLK_F8:
        {
            if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
            {
                modSetPos(-1, 32);
            }
        }
        break;

        case SDLK_F9:
        {
            if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
            {
                modSetPos(-1, 48);
            }
        }
        break;

        case SDLK_F10:
        {
            if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
            {
                modSetPos(-1, 63);
            }
        }
        break;

        case SDLK_LSHIFT:
        {
            input.keyb.leftShiftKeyDown = true;
        }
        break;

        case SDLK_RSHIFT:
        {
            if (editor.ui.exitScreenShown == false)
            {
                editor.playMode = PLAY_MODE_PATTERN;
                modPlay();

                editor.currentMode = MODE_RECORD;
                pointerSetMode(POINTER_MODE_EDIT, true);
                setStatusMessage("ALL RIGHT", true);
            }
        }
        break;

        case SDLK_LCTRL:
        {
            input.keyb.leftControlKeyDown = true;
        }
        break;

        case SDLK_LALT:
        {
            input.keyb.leftAltKeyDown = true;
        }
        break;

        case SDLK_RALT:
        {
            if (editor.ui.exitScreenShown == false)
            {
                editor.playMode = PLAY_MODE_NORMAL;
                modPlay();

                editor.currentMode = MODE_PLAY;
                pointerSetMode(POINTER_MODE_PLAY, true);
                setStatusMessage("ALL RIGHT", true);
            }
        }
        break;

        case SDLK_MENU:
        {
            if (editor.ui.exitScreenShown == false)
            {
                editor.playMode = PLAY_MODE_PATTERN;
                modPlay();

                editor.currentMode = MODE_PLAY;
                pointerSetMode(POINTER_MODE_PLAY, true);
                setStatusMessage("ALL RIGHT", true);
            }
        }
        break;

        case SDLK_TAB:
        {
            if (input.keyb.leftShiftKeyDown == true)
            {
                movePatCurPrevCh();
            }
            else
            {
                movePatCurNextCh();
            }
        }
        break;

        case SDLK_KP0:
        {
            editor.currentSample = 0;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP1:
        {
            editor.currentSample = 12 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP2:
        {
            editor.currentSample = 13 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP3:
        {
            editor.currentSample = 14 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP4:
        {
            editor.currentSample = 8 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP5:
        {
            editor.currentSample = 9 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP6:
        {
            editor.currentSample = 10 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP7:
        {
            editor.currentSample = 4 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP8:
        {
            editor.currentSample = 5 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP9:
        {
            editor.currentSample = 6 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP_ENTER:
        {
            editor.currentSample = 15 + sampleOffset;
            setupSampler(modEntry);
            input.keyb.keypadEnterKeyDown = true;
        }
        break;

        case SDLK_KP_PLUS:
        {
            editor.currentSample = 11 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP_MINUS:
        {
            editor.currentSample = 7 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP_MULTIPLY:
        {
            editor.currentSample = 3 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_KP_DIVIDE:
        {
            editor.currentSample = 2 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_NUMLOCK:
        {
            editor.currentSample = 1 + sampleOffset;
            setupSampler(modEntry);
        }
        break;

        case SDLK_DOWN:
        {
            if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
            {
                if (editor.ui.diskOpScreenShown == true)
                {
                    editor.diskop.scrollOffset++;
                    editor.diskop.forceUpdate = true;
                }
                else if (editor.ui.samplerScreenShown == false)
                {
                    if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
                    {
                        if (modEntry->currRow == 63)
                        {
                            modEntry->currRow = 0;
                        }
                        else
                        {
                            modEntry->currRow++;
                        }

                        modSetPos(-1, modEntry->currRow);
                    }
                }

                input.keyb.lastCursorKey = SDLK_DOWN;
                input.keyb.cursorKeyRepeatCounter = 0;
                input.keyb.cursorKeyPressed = true;
            }
        }
        break;

        case SDLK_UP:
        {
            if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
            {
                if (editor.ui.diskOpScreenShown == true)
                {
                    if (editor.diskop.scrollOffset > 0)
                    {
                        editor.diskop.scrollOffset--;
                    }

                    editor.diskop.forceUpdate = true;
                }
                else if (editor.ui.samplerScreenShown == false)
                {
                    if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
                    {
                        if (modEntry->currRow == 0)
                        {
                            modEntry->currRow = 63;
                        }
                        else
                        {
                            modEntry->currRow--;
                        }

                        modSetPos(-1, modEntry->currRow);
                    }
                }

                input.keyb.lastCursorKey = SDLK_UP;
                input.keyb.cursorKeyRepeatCounter = 0;
                input.keyb.cursorKeyPressed = true;
            }
        }
        break;

        // move pattern cursor left
        case SDLK_LEFT:
        {
            if (input.keyb.leftControlKeyDown == true)
            {
                if (editor.currentSample > 0)
                {
                    editor.currentSample--;
                }

                setupSampler(modEntry);
            }
            else if (input.keyb.leftShiftKeyDown == true)
            {
                input.keyb.cursorKeyPressed = true;

                if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                {
                    if (modEntry->currOrder > 0)
                    {
                        modSetPos(modEntry->currOrder - 1, -1);
                    }
                }

                input.keyb.lastCursorKey = SDLK_LEFT;
            }
            else if (input.keyb.leftAltKeyDown == true)
            {
                if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                {
                    if (modEntry->currOrder > 0)
                    {
                        modSetPos(modEntry->currOrder - 1, -1);
                    }
                }
            }
            else
            {
                movePatCurLeft();

                input.keyb.lastCursorKey = SDLK_LEFT;
                input.keyb.cursorKeyPressed = true;
            }
        }
        break;

        // move pattern cursor right
        case SDLK_RIGHT:
        {
            if (input.keyb.leftControlKeyDown == true)
            {
                if (editor.currentSample < 30)
                {
                    editor.currentSample++;
                }

                setupSampler(modEntry);
            }
            else if (input.keyb.leftShiftKeyDown == true)
            {
                input.keyb.cursorKeyPressed = true;

                if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                {
                    if (modEntry->currOrder < (modEntry->head.orderCount - 1))
                    {
                        modSetPos(modEntry->currOrder + 1, -1);
                    }
                }

                input.keyb.lastCursorKey = SDLK_RIGHT;
            }
            else if (input.keyb.leftAltKeyDown == true)
            {
                if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                {
                    if (modEntry->currOrder < (modEntry->head.orderCount - 1))
                    {
                        modSetPos(modEntry->currOrder + 1, -1);
                    }
                }
            }
            else
            {
                movePatCurRight();

                input.keyb.lastCursorKey = SDLK_RIGHT;
                input.keyb.cursorKeyPressed = true;
            }
        }
        break;

        case SDLK_a:
        {
            if (input.keyb.leftControlKeyDown == true)
            {
                if (input.keyb.leftShiftKeyDown == true)
                {
                    editor.muted[0] = true;
                    editor.muted[1] = true;
                    editor.muted[2] = true;
                    editor.muted[3] = true;
                    editor.muted[editor.cursor.channel] = false;

                    break;
                }

                editor.muted[editor.cursor.channel] = !editor.muted[editor.cursor.channel];
            }
            else if (input.keyb.leftAltKeyDown == true)
            {
                editor.muted[2] = !editor.muted[2]; // toggle channel 3
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_c:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.muted[2] = !editor.muted[2]; // toggle channel 3
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_d:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.ui.diskOpScreenShown = !editor.ui.diskOpScreenShown;

                if (editor.ui.diskOpScreenShown == true)
                {
                    editor.diskop.forceUpdate = true;
                }
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_q:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.ui.exitScreenShown = true;

                pointerSetMode(POINTER_MODE_MSG1, false);
                setStatusMessage("REALLY QUIT ?", false);
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_s:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.ui.samplerScreenShown = !editor.ui.samplerScreenShown;

                if (editor.ui.samplerScreenShown == true)
                {
                    setupSampler(modEntry);
                }
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_v:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.muted[3] = !editor.muted[3]; // toggle channel 4
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_x:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.muted[1] = !editor.muted[1]; // toggle channel 2
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        case SDLK_z:
        {
            if (input.keyb.leftAltKeyDown == true)
            {
                editor.muted[0] = !editor.muted[0]; // toggle channel 1
            }
            else
            {
                handleEditKeys(keyEntry, modEntry);
            }
        }
        break;

        default:
        {
            handleEditKeys(keyEntry, modEntry);
        }
        break;
    }
}

void movePatCurPrevCh(void)
{
    signed int pos = ((editor.cursor.pos + 5) / 6) - 1;
    if (pos < 0)
    {
        editor.cursor.pos = 3 * 6;
        editor.cursor.mode = CURSOR_NOTE;
    }
    else
    {
        editor.cursor.pos = pos * 6;
        editor.cursor.mode = CURSOR_NOTE;
    }
}

void movePatCurNextCh(void)
{
    int pos = (editor.cursor.pos / 6) + 1;
    if (pos == 4)
    {
        editor.cursor.pos = 0;
        editor.cursor.mode = CURSOR_NOTE;
    }
    else
    {
        editor.cursor.pos = pos * 6;
        editor.cursor.mode = CURSOR_NOTE;
    }
}

void movePatCurRight(void)
{
    if (editor.cursor.pos == 23)
    {
        editor.cursor.pos = 0;
    }
    else
    {
        editor.cursor.pos++;
    }

    if (editor.cursor.pos < 6)
    {
        editor.cursor.channel = 0;
    }
    else if (editor.cursor.pos < 12)
    {
        editor.cursor.channel = 1;
    }
    else if (editor.cursor.pos < 18)
    {
        editor.cursor.channel = 2;
    }
    else if (editor.cursor.pos < 24)
    {
        editor.cursor.channel = 3;
    }

    editor.cursor.mode = editor.cursor.pos % 6;
}

void movePatCurLeft(void)
{
    if (editor.cursor.pos == 0)
    {
        editor.cursor.pos = 23;
    }
    else
    {
        editor.cursor.pos--;
    }

    if (editor.cursor.pos < 6)
    {
        editor.cursor.channel = 0;
    }
    else if (editor.cursor.pos < 12)
    {
        editor.cursor.channel = 1;
    }
    else if (editor.cursor.pos < 18)
    {
        editor.cursor.channel = 2;
    }
    else if (editor.cursor.pos < 24)
    {
        editor.cursor.channel = 3;
    }

    editor.cursor.mode = editor.cursor.pos % 6;
}

void handleArrowKeys(SDLKey keyEntry, MODULE *modEntry)
{
    if (input.keyb.cursorKeyPressed == true)
    {
        input.keyb.cursorKeyRepeatCounter++;
        input.keyb.cursorKeyRepeatCounter &= 0x000000FF;

        switch (keyEntry)
        {
            case SDLK_LEFT:
            {
                if (input.keyb.leftShiftKeyDown == true)
                {
                    if (input.keyb.cursorKeyRepeatCounter >= 6)
                    {
                        input.keyb.cursorKeyRepeatCounter = 0;

                        if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                        {
                            if (modEntry->currOrder > 0)
                            {
                                modSetPos(modEntry->currOrder - 1, -1);
                            }
                        }
                    }
                }
                else
                {
                    if (input.keyb.cursorKeyRepeatCounter >= 6)
                    {
                        input.keyb.cursorKeyRepeatCounter = 0;
                        movePatCurLeft();
                    }
                }
            }
            break;

            case SDLK_RIGHT:
            {
                if (input.keyb.leftShiftKeyDown == true)
                {
                    if (input.keyb.cursorKeyRepeatCounter >= 6)
                    {
                        input.keyb.cursorKeyRepeatCounter = 0;

                        if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                        {
                            if (modEntry->currOrder < (modEntry->head.orderCount - 1))
                            {
                                modSetPos(modEntry->currOrder + 1, -1);
                            }
                        }
                    }
                }
                else
                {
                    if (input.keyb.cursorKeyRepeatCounter >= 6)
                    {
                        input.keyb.cursorKeyRepeatCounter = 0;
                        movePatCurRight();
                    }
                }
            }
            break;

            case SDLK_UP:
            {
                if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                {
                    if (editor.ui.diskOpScreenShown == true)
                    {
                        if (input.keyb.cursorKeyRepeatCounter >= 1)
                        {
                            input.keyb.cursorKeyRepeatCounter = 0;

                            if (editor.diskop.scrollOffset > 0)
                            {
                                editor.diskop.scrollOffset--;
                            }

                            editor.diskop.forceUpdate = true;
                        }
                    }
                    else if (editor.ui.samplerScreenShown == false)
                    {
                        if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
                        {
                            int repeats_needed = 6;

                            if (input.keyb.leftShiftKeyDown == true)
                            {
                                repeats_needed = 3;
                            }

                            if (input.keyb.leftAltKeyDown == true)
                            {
                                repeats_needed = 1;
                            }

                            if (input.keyb.cursorKeyRepeatCounter >= repeats_needed)
                            {
                                int row = modEntry->currRow;

                                input.keyb.cursorKeyRepeatCounter = 0;

                                if (row == 0)
                                {
                                    row = 63;
                                }
                                else
                                {
                                    row--;
                                }

                                modSetPos(-1, row);
                            }
                        }
                    }
                }
            }
            break;

            case SDLK_DOWN:
            {
                if ((editor.ui.exitScreenShown == false) && (editor.ui.clearScreenShown == false))
                {
                    if (editor.ui.diskOpScreenShown == true)
                    {
                        if (input.keyb.cursorKeyRepeatCounter >= 1)
                        {
                            input.keyb.cursorKeyRepeatCounter = 0;

                            editor.diskop.scrollOffset++;
                            editor.diskop.forceUpdate = true;
                        }
                    }
                    else if (editor.ui.samplerScreenShown == false)
                    {
                        if ((editor.currentMode != MODE_PLAY) && (editor.currentMode != MODE_RECORD))
                        {
                            int repeats_needed = 6;

                            if (input.keyb.leftShiftKeyDown == true)
                            {
                                repeats_needed = 3;
                            }

                            if (input.keyb.leftAltKeyDown == true)
                            {
                                repeats_needed = 1;
                            }

                            if (input.keyb.cursorKeyRepeatCounter >= repeats_needed)
                            {
                                int row = modEntry->currRow;

                                input.keyb.cursorKeyRepeatCounter = 0;

                                if (row == 63)
                                {
                                    row = 0;
                                }
                                else
                                {
                                    row++;
                                }

                                modSetPos(-1, row);
                            }
                        }
                    }
                }
            }
            break;

            default:
            break;
        }
    }
    else
    {
        input.keyb.cursorKeyRepeatCounter = 0;
    }
}
