#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pt_header.h"
#include "pt_config.h"

#define DEFAULT_CONFIG_FILE "protracker.ini"

void loadConfig(PTCONFIG *config)
{
    char *configBuffer;
    unsigned long configFileSize;
    FILE *configFile;

    // set standard config values first
    config->region = AMIGA_PAL;
    config->blit2x = true;
    config->soundBufferSize = 882;
    config->soundFrequency = 44100;
    config->stereoSeparation = 30;

    configFile = fopen(DEFAULT_CONFIG_FILE, "r");
    if (configFile == NULL)
    {
        fprintf(stderr, "Could not open protracker.ini, using default settings.\n");
        return;
    }

    fseek(configFile, 0, SEEK_END);
    configFileSize = ftell(configFile);
    rewind(configFile);

    configBuffer = (char *)malloc(configFileSize);
    fread(configBuffer, 1, configFileSize, configFile);
    fclose(configFile);

    configBuffer = strtok(configBuffer, "\n");
    while (configBuffer != NULL)
    {
        // COMMENT OR CATEGORY
        if ((*configBuffer == ';') || (*configBuffer == '['))
        {
            configBuffer = strtok(NULL, "\n");

            continue;
        }

        // REGION
        else if (strncmp(configBuffer, "REGION=", 7) == 0)
        {
            if (strncmp(&configBuffer[7], "PAL", 3) == 0)
            {
                config->region = AMIGA_PAL;
            }
            else if (strncmp(&configBuffer[7], "NTSC", 4) == 0)
            {
                config->region = AMIGA_NTSC;
            }
        }

        // 2XSCALE
        else if (strncmp(configBuffer, "2XSCALE=", 8) == 0)
        {
            if (strncmp(&configBuffer[8], "TRUE", 4) == 0)
            {
                config->blit2x = true;
            }
            else if (strncmp(&configBuffer[8], "FALSE", 5) == 0)
            {
                config->blit2x = false;
            }
        }

        // FREQUENCY
        else if (strncmp(configBuffer, "FREQUENCY=", 10) == 0)
        {
            if (configBuffer[10] != 0)
            {
                config->soundFrequency = atoi(&configBuffer[10]);
            }
        }

        // BUFFERSIZE
        else if (strncmp(configBuffer, "BUFFERSIZE=", 11) == 0)
        {
            if (configBuffer[11] != 0)
            {
                config->soundBufferSize = atoi(&configBuffer[11]);
            }
        }

        // STEREOSEPARATION
        else if (strncmp(configBuffer, "STEREOSEPARATION=", 17) == 0)
        {
            if (configBuffer[17] != 0)
            {
                config->stereoSeparation = atoi(&configBuffer[17]);
            }
        }

        configBuffer = strtok(NULL, "\n");
    }

    free(configBuffer);

    // sanity check of some config elements
    if (config->soundFrequency < 11025)
    {
        config->soundFrequency = 11025;
    }
    else if (config->soundFrequency > 96000)
    {
        config->soundFrequency = 96000;
    }

    if (config->soundBufferSize < 512)
    {
        config->soundBufferSize = 512;
    }
    else if (config->soundBufferSize > 4096)
    {
        config->soundBufferSize = 4096;
    }

    if (config->stereoSeparation < 0)
    {
        config->stereoSeparation = 0;
    }
    else if (config->stereoSeparation > 100)
    {
        config->stereoSeparation = 100;
    }
}
