#ifdef WIN32 // this file is for WIN32 port only

// dirent for MSVC++ from http://softagalleria.net/dirent.php

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include "pt_dirent.h"

DIR *opendir(const char *dirname)
{
    DIR *dirp;

    if (dirname == NULL)
    {
        DIRENT_SET_ERRNO(ENOENT);

        return NULL;
    }

    if ((strlen(dirname) + 3) >= MAX_PATH)
    {
        DIRENT_SET_ERRNO(ENAMETOOLONG);

        return NULL;
    }

    dirp = (DIR *)malloc(sizeof (struct DIR));
    if (dirp != NULL)
    {
        int error;

        if (GetFullPathNameA(dirname, MAX_PATH, dirp->patt, NULL))
        {
            char *p;

            p = strchr(dirp->patt, '\0');
            if ((dirp->patt < p)  &&  (*(p - 1) != '\\')  &&  (*(p - 1) != ':'))
            {
                *p++ = '\\';
            }

            *p++ = '*';
            *p = '\0';

            dirp->search_handle = FindFirstFileA(dirp->patt, &dirp->find_data);
            if (dirp->search_handle != INVALID_HANDLE_VALUE)
            {
                dirp->cached = 1;
                error = 0;
            }
            else
            {
                DIRENT_SET_ERRNO(ENOENT);
                error = 1;
            }
        }
        else
        {
            DIRENT_SET_ERRNO(ENOMEM);
            error = 1;
        }

        if (error)
        {
            free (dirp);
            dirp = NULL;
        }
    }

    return dirp;
}

struct dirent *readdir(DIR *dirp)
{
    DWORD attr;
    if (dirp == NULL)
    {
        DIRENT_SET_ERRNO(EBADF);
        return NULL;
    }

    if (dirp->cached != 0)
    {
        dirp->cached = 0;
    }
    else
    {
        if (dirp->search_handle == INVALID_HANDLE_VALUE)
        {
            return NULL;
        }

        if (FindNextFileA(dirp->search_handle, &dirp->find_data) == FALSE)
        {
            FindClose(dirp->search_handle);
            dirp->search_handle = INVALID_HANDLE_VALUE;

            return NULL;
        }
    }

    DIRENT_STRNCPY(dirp->curentry.d_name, dirp->find_data.cFileName, sizeof (dirp->curentry.d_name));
    dirp->curentry.d_name[MAX_PATH] = '\0';

    dirp->curentry.d_namlen = strlen(dirp->curentry.d_name);

    attr = dirp->find_data.dwFileAttributes;
    if ((attr & FILE_ATTRIBUTE_DEVICE) != 0)
    {
        dirp->curentry.d_type = DT_CHR;
    }
    else if ((attr & FILE_ATTRIBUTE_DIRECTORY) != 0)
    {
        dirp->curentry.d_type = DT_DIR;
    }
    else
    {
        dirp->curentry.d_type = DT_REG;
    }

    return &dirp->curentry;
}

int closedir(DIR *dirp)
{
    if (dirp == NULL)
    {
        DIRENT_SET_ERRNO(EBADF);
        return -1;
    }

    if (dirp->search_handle != INVALID_HANDLE_VALUE)
    {
        FindClose(dirp->search_handle);
        dirp->search_handle = INVALID_HANDLE_VALUE;
    }

    free(dirp);

    return 0;
}

void rewinddir(DIR* dirp)
{
    if (dirp != NULL)
    {
        if (dirp->search_handle != INVALID_HANDLE_VALUE)
        {
            FindClose(dirp->search_handle);
        }

        dirp->search_handle = FindFirstFileA(dirp->patt, &dirp->find_data);
        if (dirp->search_handle != INVALID_HANDLE_VALUE)
        {
            dirp->cached = 1;
        }
        else
        {
            dirp->cached = 0;
        }
    }
}

#endif
