#ifndef __PT_DIRENT_H
#define __PT_DIRENT_H

#ifdef WIN32
#include <windows.h>
#include <errno.h>
#include <sys/stat.h>
#include <limits.h>

// dirent for MSC++ from http://softagalleria.net/dirent.php

#if defined(_MSC_VER) && !defined(S_IREAD)
#define S_IFMT   _S_IFMT                      // file type mask
#define S_IFDIR  _S_IFDIR                     // directory
#define S_IFCHR  _S_IFCHR                     // character device
#define S_IFFIFO _S_IFFIFO                    // pipe
#define S_IFREG  _S_IFREG                     // regular file
#define S_IREAD  _S_IREAD                     // read permission
#define S_IWRITE _S_IWRITE                    // write permission
#define S_IEXEC  _S_IEXEC                     // execute permission
#endif

#ifndef S_IFBLK
#define S_IFBLK   0                            // block device
#endif

#ifndef S_IFLNK
#define S_IFLNK   0                            // link
#endif

#ifndef S_IFSOCK
#define S_IFSOCK  0                            // socket
#endif

#if defined(_MSC_VER)
#define S_IRUSR  S_IREAD                      // read, user
#define S_IWUSR  S_IWRITE                     // write, user
#define S_IXUSR  0                            // execute, user
#define S_IRGRP  0                            // read, group
#define S_IWGRP  0                            // write, group
#define S_IXGRP  0                            // execute, group
#define S_IROTH  0                            // read, others
#define S_IWOTH  0                            // write, others
#define S_IXOTH  0                            // execute, others
#endif

#ifndef _DIRENT_HAVE_D_TYPE
#define _DIRENT_HAVE_D_TYPE
#endif

#ifndef DT_UNKNOWN
#define DT_UNKNOWN 0
#endif

#ifndef DT_REG
#define DT_REG      S_IFREG
#endif

#ifndef DT_DIR
#define DT_DIR      S_IFDIR
#endif

#ifndef DT_FIFO
#define DT_FIFO     S_IFFIFO
#endif

#ifndef DT_SOCK
#define DT_SOCK     S_IFSOCK
#endif

#ifndef DT_CHR
#define DT_CHR      S_IFCHR
#endif

#ifndef DT_BLK
#define DT_BLK      S_IFBLK
#endif

#ifndef IFTODT
#define IFTODT(mode) ((mode) & S_IFMT)
#endif

#ifndef DTTOIF
#define DTTOIF(type) (type)
#endif

#ifndef S_ISFIFO
#define	S_ISFIFO(mode) (((mode) & S_IFMT) == S_IFFIFO)
#endif

#ifndef S_ISDIR
#define	S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
#endif

#ifndef S_ISREG
#define	S_ISREG(mode) (((mode) & S_IFMT) == S_IFREG)
#endif

#ifndef S_ISLNK
#define	S_ISLNK(mode) (((mode) & S_IFMT) == S_IFLNK)
#endif

#ifndef S_ISSOCK
#define	S_ISSOCK(mode) (((mode) & S_IFMT) == S_IFSOCK)
#endif

#ifndef S_ISCHR
#define	S_ISCHR(mode) (((mode) & S_IFMT) == S_IFCHR)
#endif

#ifndef S_ISBLK
#define	S_ISBLK(mode) (((mode) & S_IFMT) == S_IFBLK)
#endif

typedef struct dirent
{
    char d_name[MAX_PATH + 1];                  // File name
    size_t d_namlen;                            // Length of name without \0
    int d_type;                                 // File type
} dirent;

typedef struct DIR
{
    dirent           curentry;                  // Current directory entry
    WIN32_FIND_DATAA find_data;                 // Private file data
    int              cached;                    // True if data is valid
    HANDLE           search_handle;             // Win32 search handle
    char             patt[MAX_PATH + 3];        // Initial directory name
} DIR;

#if defined(_MSC_VER) && _MSC_VER >= 1400
#define DIRENT_STRNCPY(dest,src,size) strncpy_s((dest), (size), (src), _TRUNCATE)
#else
#define DIRENT_STRNCPY(dest,src,size) strncpy((dest), (src), (size))
#endif

#if defined(_MSC_VER)
#define DIRENT_SET_ERRNO(x) _set_errno(x)
#else
#define DIRENT_SET_ERRNO(x) (errno = (x))
#endif

DIR *opendir(const char *dirname);
struct dirent *readdir(DIR *dirp);
int closedir(DIR *dirp);
void rewinddir(DIR *dirp);
#else
#include <errno.h>
#include <sys/stat.h>
#include <limits.h>
#include <dirent.h>
#endif

#endif
