#include <SDL/SDL.h>
#include "pt_header.h"
#include "pt_tables.h"
#include "pt_audio.h"
#include "pt_helpers.h"

static const short singlePeriodTable[36] =
{
    856, 808, 762, 720, 678, 640, 604, 570, 538, 508, 480, 453,
    428, 404, 381, 360, 339, 320, 302, 285, 269, 254, 240, 226,
    214, 202, 190, 180, 170, 160, 151, 143, 135, 127, 120, 113
};

static const char amigaKeyboardTable[64] =
{
    '1', '2', '3', '4', '5', '6', '7', '8', '9', '-', '=', '\\', '*', '0', '`',
    'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', '*', '1', '2', '3',
    'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', '\'', '#', '\0', '4', '5', '6',
    '<', 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', '\0', '.', '7', '8', '9', ' '
};

static const char keyboardScaleTable[39] =
{
    49,33,50,34,51,52,36,53,37,54,38,55,56,40,57,
    41,58,16,2,17,3,18,19,5,20,6,21,7,22,23,9,24,
    10,25,26,12,27,70,70
};

static const char keyboardTransitionTable[2][39] =
{
    // high octave (default)
    {
        12,13,14,15,16,17,18,19,20,21,22,23,24,25,
        26,27,28,24,25,26,27,28,29,30,31,32,33,34,
        35,36,36,36,36,36,36,36,36,36,36
    },

    // low octave
    {
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,
        14,15,16,12,13,14,15,16,17,18,19,20,21,22,
        23,24,25,26,27,28,29,30,31,36,36
    }
};

void handleEditKeys(SDLKey keyEntry, MODULE *modEntry)
{
    int i;
    int numberKey, hexkey, key, rawNote, rawKey;
    int noteFound;
    int tempRow, tempPeriod;
    int cleanPeriod;
    int keyTransition;
    unsigned int scaledVol, scaledVol2;
    unsigned int scaledNote;
    int lookUp;

    MODULE_SAMPLE *s;
    modnote_t *note;

    // are we editing a note, or other stuff?
    if (editor.cursor.mode != CURSOR_NOTE)
    {
        if ((editor.currentMode == MODE_EDIT) || (editor.currentMode == MODE_RECORD))
        {
            note = &modEntry->patterns[modEntry->currPattern][(modEntry->currRow * PAULA_CHANNELS) + editor.cursor.channel];

            numberKey = ((keyEntry > 47) && (keyEntry <  58)) ? keyEntry - 48 : -1;
            hexkey = ((keyEntry > 96) && (keyEntry < 103)) ? keyEntry - 87 : -1;
            key = -1;

            if (numberKey != -1)
            {
                if (key == -1)
                {
                    key = 0;
                }

                key += numberKey;
            }

            if (hexkey != -1)
            {
                if (key == -1)
                {
                    key = 0;
                }

                key += hexkey;
            }

            switch (editor.cursor.mode)
            {
                case CURSOR_SAMPLE1:
                {
                    if ((key != -1) && (key < 2))
                    {
                        note->sample = (char)((note->sample % 0x10) | (key << 4));

                        if (editor.currentMode != MODE_RECORD)
                        {
                            // increase row
                            tempRow = modEntry->currRow + 1;
                            if (tempRow > 63)
                            {
                                tempRow = 0;
                            }

                            modSetPos(-1, tempRow);
                        }
                    }
                }
                break;

                case CURSOR_SAMPLE2:
                {
                    if ((key != -1) && (key < 16))
                    {
                        note->sample = (char)((note->sample & 16) | key);

                        if (editor.currentMode != MODE_RECORD)
                        {
                            // increase row
                            tempRow = modEntry->currRow + 1;
                            if (tempRow > 63)
                            {
                                tempRow = 0;
                            }

                            modSetPos(-1, tempRow);
                        }
                    }
                }
                break;

                case CURSOR_CMD:
                {
                    if ((key != -1) && (key < 16))
                    {
                        note->command = (char)key;

                        if (editor.currentMode != MODE_RECORD)
                        {
                            // increase row
                            tempRow = modEntry->currRow + 1;
                            if (tempRow > 63)
                            {
                                tempRow = 0;
                            }

                            modSetPos(-1, tempRow);
                        }
                    }
                }
                break;

                case CURSOR_PARAM1:
                {
                    if ((key != -1) && (key < 16))
                    {
                        note->param = (char)((note->param % 0x10) | (key << 4));

                        if (editor.currentMode != MODE_RECORD)
                        {
                            // increase row
                            tempRow = modEntry->currRow + 1;
                            if (tempRow > 63)
                            {
                                tempRow = 0;
                            }

                            modSetPos(-1, tempRow);
                        }
                    }
                }
                break;

                case CURSOR_PARAM2:
                {
                    if ((key != -1) && (key < 16))
                    {
                        note->param = (char)((note->param & 0xF0) | key);

                        if (editor.currentMode != MODE_RECORD)
                        {
                            // increase row
                            tempRow = modEntry->currRow + 1;
                            if (tempRow > 63)
                            {
                                tempRow = 0;
                            }

                            modSetPos(-1, tempRow);
                        }
                    }
                }
                break;

                default:
                break;
            }
        }
    }
    else
    {
        if (keyEntry == SDLK_DELETE)
        {
            if ((editor.currentMode == MODE_EDIT) || (editor.currentMode == MODE_RECORD))
            {
                note = &modEntry->patterns[modEntry->currPattern][(modEntry->currRow * PAULA_CHANNELS) + editor.cursor.channel];
                note->sample = 0;
                note->period = 0;

                if (editor.currentMode != MODE_RECORD)
                {
                    // increase row
                    tempRow = modEntry->currRow + 1;
                    if (tempRow > 63)
                    {
                        tempRow = 0;
                    }

                    modSetPos(-1, tempRow);
                }
            }
        }
        else
        {
            rawNote = 0;
            rawKey = 0;
            noteFound = false;

            // look up note by raw key
            for (i = 0; i < 64; ++i)
            {
                if (keyEntry == amigaKeyboardTable[i])
                {
                    rawKey = i + 1;
                    break;
                }
            }

            for (i = 0; i < 39; ++i)
            {
                if (rawKey == keyboardScaleTable[i])
                {
                    rawNote = i;
                    noteFound = true;

                    break;
                }
            }
        }

        // was a note found in range?
        if (noteFound == true)
        {
            s = &modEntry->samples[editor.currentSample];

            tempPeriod = amigaPeriods[s->fineTune][(int)keyboardTransitionTable[editor.keyOctave][rawNote]];
            keyTransition = keyboardTransitionTable[editor.keyOctave][rawNote];

            if (keyTransition >= 36)
            {
                return; // unused key pressed, let's return
            }

            cleanPeriod = singlePeriodTable[keyTransition];

            // insert note and sample number
            if ((editor.currentMode == MODE_EDIT) || (editor.currentMode == MODE_RECORD))
            {
                note = &modEntry->patterns[modEntry->currPattern][(modEntry->currRow * PAULA_CHANNELS) + editor.cursor.channel];
                note->sample = (char)editor.currentSample + 1;
                note->period = (short)cleanPeriod;

                if (editor.currentMode != MODE_RECORD)
                {
                    // increase row
                    tempRow = modEntry->currRow + 1;
                    if (tempRow > 63)
                    {
                        tempRow = 0;
                    }

                    modSetPos(-1, tempRow);
                }
            }

            // play current sample
            if (editor.currentMode != MODE_PLAY)
            {
                if (s->length > 0)
                {
                    modEntry->channels[editor.cursor.channel].sample = editor.currentSample + 1;

                    mixerSetChSource(editor.cursor.channel, modEntry->sampleData + s->offset, s->length, s->loopStart, s->loopLength, 0);
                    mixerSetChRate(editor.cursor.channel, frequencyTable[tempPeriod]);
                    mixerSetChVol(editor.cursor.channel, s->volume);

                    // Update spectrum bars if neeeded
                    if (editor.ui.visualizerMode == VISUAL_SPECTRUM)
                    {
                        // PT doesn't check if the channel is muted here, kind of odd
                        scaledVol = s->volume;
                        scaledVol = (scaledVol * 256) / 682;
                        scaledVol2 = scaledVol / 2;

                        scaledNote = 743 - (tempPeriod - 113);
                        scaledNote = (scaledNote * scaledNote) / 25093;
                        if (scaledNote > 22)
                        {
                            scaledNote = 22;
                        }

                        editor.spectrumVolumes[scaledNote] += scaledVol;
                        if (editor.spectrumVolumes[scaledNote] > 35)
                        {
                            editor.spectrumVolumes[scaledNote] = 35;
                        }

                        if (scaledNote > 0)
                        {
                            lookUp = scaledNote - 1;

                            editor.spectrumVolumes[lookUp] += scaledVol2;
                            if (editor.spectrumVolumes[lookUp] > 35)
                            {
                                editor.spectrumVolumes[lookUp] = 35;
                            }
                        }

                        if (scaledNote < 22)
                        {
                            lookUp = scaledNote + 1;

                            editor.spectrumVolumes[lookUp] += scaledVol2;
                            if (editor.spectrumVolumes[lookUp] > 35)
                            {
                                editor.spectrumVolumes[lookUp] = 35;
                            }
                        }
                    }
                }
            }
        }
    }
}
