#ifndef __PT_DEFINES_H
#define __PT_DEFINES_H

#include <SDL/SDL.h>
#include "pt_config.h"

#ifdef PATH_MAX
#undef PATH_MAX
#endif

#define PATH_MAX 260

#ifndef true
#define true 1
#endif

#ifndef false
#define false 0
#endif

#define SCREEN_W 320
#define SCREEN_H 255
#define FULL_SCREEN_W 720
#define FULL_SCREEN_H 576
#define FONT_CHAR_W 8 // actual data length is 7, includes right spacing (1px column)
#define FONT_CHAR_H 5

// not including min/max + finetune
#define PT_MIN_PERIOD 113
#define PT_MAX_PERIOD 856

#define PAULA_CHANNELS 4
#define MOD_ROWS 64
#define MOD_SAMPLES 31
#define MOD_ORDERS 128

#define AMIGA_PAULA_PAL_HZ 3546895
#define AMIGA_PAULA_NTSC_HZ 3579545

enum
{
    FORMAT_MK,     // ProTracker 1.x
    FORMAT_MK2,    // ProTracker 2.x (if tune has >64 patterns)
    FORMAT_FLT4,   // StarTrekker
    FORMAT_4CHN,   // FastTracker II (only 4 channel MODs)
    FORMAT_STK,    // The Ultimate SoundTracker (15 samples)
    FORMAT_NT,     // NoiseTracker 1.0
    FORMAT_UNKNOWN
};

typedef struct modnote
{
    short period;
    unsigned char sample;
    unsigned char command;
    unsigned char param;
} modnote_t;

typedef struct
{
    char moduleTitle[21];

    unsigned char orderCount, patternCount, restartPos;

    unsigned int moduleSize;
    unsigned int totalSampleSize;

    unsigned char order[MOD_ORDERS];
    unsigned char tempo, ticks;
    unsigned char format;
} MODULE_HEADER;

typedef struct
{
    char text[23];

    int loopStart, loopLength, length;
    int offset;

    signed char fineTune;
    unsigned char volume;
} MODULE_SAMPLE;

typedef struct
{
    unsigned char chanIndex;

    int noNote;

    signed short period, tempPeriod;
    int offset, offsetTemp, offsetBugNotAdded;

    signed char patternLoopRow, patternLoopCounter;

    unsigned char sample, command, param;

    signed char volume, rawVolume;

    unsigned char flags, tempFlags, tempFlagsBackup;

    unsigned char fineTune;

    unsigned char tremoloPos, vibratoPos;

    unsigned char tremoloControl, tremoloSpeed, tremoloDepth;
    unsigned char vibratoControl, vibratoSpeed, vibratoDepth;
    unsigned char glissandoControl, glissandoSpeed;

    unsigned char invertLoopDelay, invertLoopSpeed;
    int invertLoopOffset;
} mod_channel;

typedef struct
{
    unsigned char currOrder, currPattern;
    signed char currRow; // must be signed
    unsigned char currBPM, currSpeed;

    unsigned char moduleLoaded;

    unsigned char MAX_PATTERNS;

    MODULE_HEADER head;
    MODULE_SAMPLE samples[MOD_SAMPLES];
    modnote_t *patterns[100];
    signed char *sampleData;
    mod_channel channels[PAULA_CHANNELS];
} MODULE;

void modSetPos(int order, int row);
void modStop(void);
void modPlay(void);
void modSetSpeed(int speed);
void modSetTempo(int bpm);
void modFree(MODULE *modEntry);

int audioOpen(const PTCONFIG *config);
void audioClose(void);
void setPlayerModSource(MODULE *modEntry);
void setMainModSource(MODULE *modEntry);

enum
{
    SCREEN_ALL = 0,
    SCREEN_MAINSCREEN = 1,
    SCREEN_DISKOP = 2,
    SCREEN_SAMPLER = 4,
    SCREEN_QUIT = 8,
    SCREEN_CLEAR = 16
};

enum
{
    VISUAL_ABOUT_BOX = 0,
    VISUAL_QUADRASCOPE = 1,
    VISUAL_SPECTRUM = 2
};

enum
{
    MODE_IDLE = 0,
    MODE_EDIT = 1,
    MODE_PLAY = 2,
    MODE_RECORD = 3
};

enum
{
    CURSOR_NOTE    = 0,
    CURSOR_SAMPLE1 = 1,
    CURSOR_SAMPLE2 = 2,
    CURSOR_CMD     = 3,
    CURSOR_PARAM1  = 4,
    CURSOR_PARAM2  = 5
};

enum
{
    PLAY_MODE_NORMAL = 0,
    PLAY_MODE_PATTERN = 1
};

enum
{
    OCTAVE_HIGH = 0,
    OCTAVE_LOW = 1
};


enum
{
    AMIGA_PAL = 0,
    AMIGA_NTSC = 1
};

struct audio_t
{
    int tempoTimerVal;
    int audioMixerOpen;
} audio;

struct input_t
{
    struct keyb_t
    {
        int keypadEnterKeyDown;
        int leftControlKeyDown;
        int leftShiftKeyDown;
        int leftAltKeyDown;
        int cursorKeyPressed;
        int cursorKeyRepeatCounter;
        SDLKey lastCursorKey;
    } keyb;

    struct mouse_t
    {
        int x, y;
        int leftButtonPressed;
        int rightButtonPressed;
        int buttonWaiting;
        int repeatCounter;
        int buttonWaitCounter;
        int lastClickedGUIButton;
        unsigned int errorMsgCounter;
        int buttonErrorMsgActive;
    } mouse;
} input;

struct editor_t
{
    unsigned int playTime;
    int songPlaying;
    int keyOctave;
    int currentMode;
    int currentSample;
    int playMode;
    int muted[4];
    int useLEDFilter;
    int spectrumVolumes[23];
    int vuMeterVolumes[4];
    int endianness;

    struct diskop_t
    {
        int forceUpdate;
        int numFiles;
        char currentPath[PATH_MAX];
        int scrollOffset;
    } diskop;

    struct cursor_t
    {
        int pos, mode, channel;
    } cursor;

    struct ui_t
    {
        int introScreenShown;
        int samplerScreenShown;
        int diskOpScreenShown;
        int clearScreenShown;
        int sampleDataDrawn;
        int exitScreenShown;
        int throwExit;
        int visualizerMode;
        int pointerMode;
        int previousPointerMode;
        char previousStatusMessage[17];
        char statusMessage[17];
    } ui;
} editor;

#endif
