#include <stdio.h>
#include <string.h>
#include <ctype.h> // for toupper()
#include <SDL/SDL.h>
#include "pt_header.h"
#include "pt_palette.h"
#include "pt_tables.h"

#ifdef _MSC_VER
#pragma warning(disable: 4996) // for deprecated POSIX warnings
#endif

// Please ignore this function, it's for personal
// developing use, they may make no sense to you.
int ptBMP2Header(const char *fileName)
{
    FILE *hOut = NULL;
    SDL_Surface *sBmp = NULL;
    unsigned int *pixBuf = NULL;
    char *bmpOut = strdup(fileName), *capBmpOut;
    char hOutFilename[512], *tmpOutName;

    int i, bmpLen, bmpFilenameLen = strlen(fileName);

    tmpOutName = &bmpOut[bmpFilenameLen - 4];
    if (strcmp(".bmp", tmpOutName))
    {
        free(bmpOut);
        return 0; // Please feed me a .bmp file
    }

    bmpOut[bmpFilenameLen - 4] = '\0';
    capBmpOut = strdup(bmpOut);

    sprintf(hOutFilename, "C:\\pt_gfx_%s.h", bmpOut);

    hOut = fopen(hOutFilename, "w");
    if (hOut == NULL)
    {
        free(bmpOut);
        free(capBmpOut);

        return 0;
    }

    sBmp = SDL_LoadBMP(fileName);
    if (sBmp == NULL)
    {
        free(bmpOut);
        free(capBmpOut);
        fclose(hOut);

        return 0;
    }

    bmpLen = sBmp->w * sBmp->h;
    pixBuf = (unsigned int *)sBmp->pixels;
    if (pixBuf == NULL)
    {
        free(bmpOut);
        free(capBmpOut);
        fclose(hOut);

        SDL_FreeSurface(sBmp);

        return 0;
    }

    for (i = 0; i < (bmpFilenameLen - 4); ++i)
    {
        capBmpOut[i] = toupper(capBmpOut[i]);
    }

    fprintf(hOut, "#ifndef __PT_GFX_%s_H\n", capBmpOut);
    fprintf(hOut, "#define __PT_GFX_%s_H\n\n", capBmpOut);

    fprintf(hOut, "#define %s_WIDTH %d\n", capBmpOut, sBmp->w);
    fprintf(hOut, "#define %s_HEIGHT %d\n\n", capBmpOut, sBmp->h);

    fprintf(hOut, "const unsigned int %s_bitmap[%d] =\n{\n", bmpOut, bmpLen);

    for (i = 0; i < bmpLen; ++i)
    {
        if (!(i % 8))
        {
            fputc(0x09, hOut); // tab
        }

        fprintf(hOut, "0x%08X%s", pixBuf[i], (((i % 8) == 7) || (i == (bmpLen - 1))) ? ",\n" : ", ");
    }

    fprintf(hOut, "};\n\n#endif\n");

    free(bmpOut);
    free(capBmpOut);
    fclose(hOut);

    SDL_FreeSurface(sBmp);

    return 1;
}

unsigned short fGetWordBigEndian(FILE *fileHandle)
{
    unsigned char bytes[2];
    fread(bytes, 1, 2, fileHandle);

    return (bytes[0] << 8) | bytes[1];
}

void setPrevStatusMessage(void)
{
    strcpy(editor.ui.statusMessage, editor.ui.previousStatusMessage);
}

void setStatusMessage(const char *message, int carry)
{
    if (carry == true)
    {
        strcpy(editor.ui.previousStatusMessage, message);
    }

    strcpy(editor.ui.statusMessage, message);
}

void calcRealModSize(MODULE *modEntry)
{
    int i;

    // + mod title     (20   bytes)
    //
    // ---- x31 -----------------
    // + smp.text      (22   bytes)
    // + smp.length    (2    bytes)
    // + smp.finetune  (1    byte )
    // + smp.volume    (1    byte )
    // + smp.loopstrt  (2    bytes)
    // + smp.looplen   (2    bytes)
    // =               (30   bytes)
    // --------------------------
    // =               (930  bytes)
    //
    // + order count   (1    byte )
    // + restart pos   (1    byte )
    // + order list    (128  bytes)
    // + tag           (4    bytes)
    // + 1 pattern     (1024 bytes)
    //
    // =               (2108 bytes)

    modEntry->head.moduleSize = 2108;
    modEntry->head.patternCount = 0;

    for (i = 0; i < MOD_SAMPLES; ++i)
    {
        modEntry->head.moduleSize += modEntry->samples[i].length;
    }

    for (i = 0; i < 128; ++i)
    {
        if (modEntry->head.order[i] > modEntry->head.patternCount)
        {
            modEntry->head.patternCount = modEntry->head.order[i];
        }
    }

    modEntry->head.moduleSize += (1024 * modEntry->head.patternCount);
}

void createFrequencyTable(int soundFrequency, int baseClockRegion)
{
    int i;
    unsigned int clock;

    switch (baseClockRegion)
    {
        default:
        case AMIGA_PAL:
            clock = AMIGA_PAULA_PAL_HZ;
            audio.tempoTimerVal = (soundFrequency * 125) / 50;
        break;

        case AMIGA_NTSC:
            clock = AMIGA_PAULA_NTSC_HZ;
            audio.tempoTimerVal = (soundFrequency * 125) / 60;
        break;
    }

    // Range is 108..907 (min/max note + finetune included).
    // My code does not look up any other indexes,
    // so not initializing them should be safe.
    for (i = 108; i <= 907; ++i)
    {
        frequencyTable[i] = (float)(clock / i) / (float)soundFrequency;
    }
}
