#include <math.h>
#include <time.h>
#include "pt_textout.h"

static char scrollText[] =
    "                                         " \
    "*8BITBUBSY* IS PROUD TO PRESENT ** MULTI-PLATFORM PROTRACKER **";

#define SCROLL_BUF_WIDTH (FONT_CHAR_W * sizeof (scrollText))
static unsigned int scrollBuf[SCROLL_BUF_WIDTH * FONT_CHAR_H];
static unsigned int scrollBufPos;

static int sineTable[360];
static int sineLookup, sineRef;

static int starField[55];

const static unsigned int colorTable[54] =
{
    0x00FF00B4, 0x00FF0094, 0x00FF007A, 0x00FF005C,
    0x00FF0040, 0x00FF0027, 0x00FF0012, 0x00FF0002,
    0x00FF0300, 0x00FF1200, 0x00FF2600, 0x00FF3E00,
    0x00FF5800, 0x00FF7300, 0x00FF8C00, 0x00FFAD00,
    0x00FFC400, 0x00FFDD00, 0x00FFED00, 0x00FFFD00,
    0x00F6FF00, 0x00E2FF00, 0x00D2FF00, 0x00BCFF00,
    0x00A3FF00, 0x0083FF00, 0x006BFF00, 0x0055FF00,
    0x003AFF00, 0x0024FF00, 0x0012FF00, 0x0004FF00,
    0x0000FF08, 0x0000FF17, 0x0000FF29, 0x0000FF3E,
    0x0000FF53, 0x0000FF6B, 0x0000FF84, 0x0000FF9D,
    0x0000FFB4, 0x0000FFCA, 0x0000FFE0, 0x0000FFF2,
    0x0000FFFD, 0x0000F6FF, 0x0000E4FF, 0x0000CCFF,
    0x0000B3FF, 0x000095FF, 0x00007CFF, 0x000063FF,
    0x000044FF, 0x00002BFF
};

#ifdef _MSC_VER
inline static int ftol(const float f)
{
    __asm
    {
        fld f
        push eax
        fistp dword ptr [esp]
        pop eax
    }
}
#else
#define ftol(f) (signed int)(f)
#endif

void initIntro(void)
{
    int i;

    // Seed random generator with current timestamp
    srand((unsigned int)time(NULL));

    // Generate starfield X map
    for (i = 0; i < 55; ++i)
    {
        starField[i] = 121 + (rand() % 198);
    }

    // Generate sinus table
    for (i = 0; i < 360; ++i)
    {
        sineTable[i] = 69 + ftol(20.0f * sinf((float)i * 3.141592f / 180.0f));
    }

    // Pre-blit scroller text to a buffer
    scrollerTextOut(scrollBuf, 0, 0, SCROLL_BUF_WIDTH, scrollText, 0x00FFFFFF);

    scrollBufPos = 0;
    sineLookup = 0;
    sineRef = 0;
}

void drawIntro(unsigned int *frameBuffer)
{
    int i, x, y;

    // Fill some shade pixels in the corner window
    frameBuffer[17400] = 0x00BBBBBB;
    frameBuffer[27960] = 0x00BBBBBB;
    frameBuffer[17919] = 0x00555555;
    frameBuffer[28479] = 0x00555555;

    // Fill area with black
    for (y = 45; y < 98; ++y)
    {
        int dstY = y * SCREEN_W;
        for (x = 121; x < 319; ++x)
        {
            frameBuffer[dstY + x] = 0x00000000;
        }
    }

    // Put stars
    for (y = 45; y < 98; ++y)
    {
        int starLookup = y - 45;
        frameBuffer[(y * SCREEN_W) + starField[starLookup]] = 0x00555555 * ((starLookup % 3) + 1);
    }

    // Update sinus table reference
    sineRef = (sineRef + 4) % 360;
    sineLookup = sineRef;

    // Put sinecroller buffer and apply sinuswave on Y
    for (x = 121; x < 319; ++x)
    {
        int sinY = sineTable[sineLookup];
        int srcX = ((x - 121) + scrollBufPos) % SCROLL_BUF_WIDTH;

        for (y = 0; y < FONT_CHAR_H; ++y)
        {
            if (scrollBuf[(y * SCROLL_BUF_WIDTH) + srcX])
            {
                int dstY = sinY + y;
                frameBuffer[(dstY * SCREEN_W) + x] = colorTable[dstY - 40];
            }
        }

        // Update sinus on every second pixel
        if (x & 1)
        {
            sineLookup = (sineLookup + 2) % 360;
        }
    }

    // Move stars in starfield map
    for (i = 0; i < 55; ++i)
    {
        starField[i] -= ((i % 3) + 1);
        if (starField[i] < 121)
        {
            starField[i] = 318;
        }
    }

    // Increment sinescroller buffer index
    scrollBufPos++;
}
