#include <SDL/SDL.h>
#include "pt_header.h"
#include "pt_buttons.h"
#include "pt_helpers.h"
#include "pt_palette.h"
#include "pt_diskop.h"
#include "pt_sampler.h"
#include "pt_modloader.h"

void diskOpLoadFile(const SDL_Surface *screen, int fileEntryRow, MODULE *modEntry);

void sampleUpButton(void);
void sampleDownButton(void);
void sampleFineTuneUpButton(MODULE *modEntry);
void sampleFineTuneDownButton(MODULE *modEntry);
void sampleVolumeUpButton(MODULE *modEntry);
void sampleVolumeDownButton(MODULE *modEntry);
void sampleLengthUpButton(MODULE *modEntry);
void sampleLengthDownButton(MODULE *modEntry);
void sampleRepeatUpButton(MODULE *modEntry);
void sampleRepeatDownButton(MODULE *modEntry);
void sampleRepeatLengthUpButton(MODULE *modEntry);
void sampleRepeatLengthDownButton(MODULE *modEntry);
void tempoUpButton(MODULE *modEntry);
void tempoDownButton(MODULE *modEntry);
void patternLengthUpButton(MODULE *modEntry);
void patternLengthDownButton(MODULE *modEntry);
void patternUpButton(MODULE *modEntry);
void patternDownButton(MODULE *modEntry);
void positionUpButton(MODULE *modEntry);
void positionDownButton(MODULE *modEntry);

void mouseButtonUpHandler(int mouseButton)
{
    input.mouse.buttonWaitCounter = 0;
    input.mouse.buttonWaiting = false;

    if (mouseButton == SDL_BUTTON_LEFT)
    {
        input.mouse.leftButtonPressed = false;
    }

    if (mouseButton == SDL_BUTTON_RIGHT)
    {
        input.mouse.rightButtonPressed = false;
    }
}

void mouseButtonDownHandler(int mouseX, int mouseY, const SDL_Surface *screen, int mouseButton, MODULE *modEntry)
{
    if (mouseButton == SDL_BUTTON_LEFT)
    {
        input.mouse.leftButtonPressed = true;
        input.mouse.buttonWaiting = true;
    }

    if (mouseButton == SDL_BUTTON_RIGHT)
    {
        input.mouse.rightButtonPressed = true;
    }

    if (input.mouse.leftButtonPressed == true)
    {
        int GUIButton = checkGUIButtons(mouseX, mouseY);
        if (GUIButton != -1)
        {
            switch (GUIButton)
            {
                case PTB_SUREY:
                {
                    editor.ui.exitScreenShown = false;
                    editor.ui.throwExit = true;
                }
                break;

                case PTB_SUREN:
                {
                    editor.ui.exitScreenShown = false;

                    pointerSetPreviousMode();
                    setPrevStatusMessage();
                }
                break;

                case PTB_VISUALS:
                {
                    if (editor.ui.introScreenShown == false)
                    {
                        if (editor.ui.visualizerMode == VISUAL_QUADRASCOPE)
                        {
                            editor.ui.visualizerMode = VISUAL_SPECTRUM;
                            memset(editor.spectrumVolumes, 0, 23 * sizeof (int));
                        }
                        else
                        {
                            editor.ui.visualizerMode = VISUAL_QUADRASCOPE;
                        }

                        input.mouse.lastClickedGUIButton = GUIButton;
                    }
                }
                break;

                case PTB_QUIT:
                {
                    editor.ui.exitScreenShown = true;

                    pointerSetMode(POINTER_MODE_MSG1, false);
                    setStatusMessage("REALLY QUIT ?", false);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CHAN1:
                {
                    editor.muted[0] = !editor.muted[0];
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CHAN2:
                {
                    editor.muted[1] = !editor.muted[1];
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CHAN3:
                {
                    editor.muted[2] = !editor.muted[2];
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CHAN4:
                {
                    editor.muted[3] = !editor.muted[3];
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SAMPLER:
                {
                    editor.ui.samplerScreenShown = !editor.ui.samplerScreenShown;

                    if (editor.ui.samplerScreenShown == true)
                    {
                        editor.ui.sampleDataDrawn = false;
                        setupSampler(modEntry);
                    }

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SA_EXIT:
                {
                    editor.ui.samplerScreenShown = false;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DISKOP:
                {
                    editor.ui.diskOpScreenShown = true;
                    editor.diskop.forceUpdate = true;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_FILEAREA:
                {
                    diskOpLoadFile(screen, (input.mouse.y - 46) / 6, modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_PARENT:
                {
                    diskOpGoParent();
                    editor.diskop.scrollOffset = 0;
                    editor.diskop.forceUpdate = true;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_EXIT:
                {
                    editor.ui.diskOpScreenShown = false;
                    editor.diskop.forceUpdate = false;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_SCROLLUP:
                {
                    if (editor.diskop.scrollOffset > 0)
                    {
                        editor.diskop.scrollOffset--;
                    }

                    editor.diskop.forceUpdate = true;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_SCROLLTOP:
                {
                    editor.diskop.scrollOffset = 0;
                    editor.diskop.forceUpdate = true;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_SCROLLDOWN:
                {
                    if (editor.diskop.scrollOffset < editor.diskop.numFiles)
                    {
                        editor.diskop.scrollOffset++;
                    }

                    editor.diskop.forceUpdate = true;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_DO_SCROLLBOT:
                {
                    editor.diskop.scrollOffset = editor.diskop.numFiles;
                    editor.diskop.forceUpdate = true;
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_STOP:
                {
                    editor.playMode = PLAY_MODE_NORMAL;

                    modStop();

                    editor.currentMode = MODE_IDLE;
                    pointerSetMode(POINTER_MODE_IDLE, true);
                    setStatusMessage("ALL RIGHT", true);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_PLAY:
                {
                    editor.playMode = PLAY_MODE_NORMAL;

                    modPlay();

                    editor.currentMode = MODE_PLAY;
                    pointerSetMode(POINTER_MODE_PLAY, true);
                    setStatusMessage("ALL RIGHT", true);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_PATTERN:
                {
                    editor.playMode = PLAY_MODE_PATTERN;

                    modPlay();

                    editor.currentMode = MODE_PLAY;
                    pointerSetMode(POINTER_MODE_PLAY, true);
                    setStatusMessage("ALL RIGHT", true);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_EDIT:
                {
                    if (editor.ui.samplerScreenShown == false)
                    {
                        editor.playMode = PLAY_MODE_NORMAL;

                        modStop();

                        editor.currentMode = MODE_EDIT;
                        pointerSetMode(POINTER_MODE_EDIT, true);
                        setStatusMessage("ALL RIGHT", true);

                        input.mouse.lastClickedGUIButton = GUIButton;
                    }
                }
                break;

                case PTB_RECORD:
                {
                    if (editor.ui.samplerScreenShown == false)
                    {
                        editor.playMode = PLAY_MODE_PATTERN;

                        modPlay();

                        editor.currentMode = MODE_RECORD;
                        pointerSetMode(POINTER_MODE_EDIT, true);
                        setStatusMessage("ALL RIGHT", true);

                        input.mouse.lastClickedGUIButton = GUIButton;
                    }
                }
                break;

                case PTB_CLEAR:
                {
                    editor.ui.clearScreenShown = true;

                    setStatusMessage("PLEASE SELECT", false);
                    pointerSetMode(POINTER_MODE_MSG1, false);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CLEARSONG:
                {
                    editor.ui.clearScreenShown = false;
                    editor.playMode = PLAY_MODE_NORMAL;

                    modStop();
                    clearSong();

                    editor.currentMode = MODE_IDLE;
                    pointerSetMode(POINTER_MODE_IDLE, true);
                    setStatusMessage("ALL RIGHT", true);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CLEARSAMPLES:
                {
                    editor.ui.clearScreenShown = false;
                    editor.playMode = PLAY_MODE_NORMAL;

                    modStop();
                    clearSamples();

                    editor.currentMode = MODE_IDLE;
                    pointerSetMode(POINTER_MODE_IDLE, true);
                    setStatusMessage("ALL RIGHT", true);

                    setupSampler(modEntry);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CLEARALL:
                {
                    editor.ui.clearScreenShown = false;
                    editor.playMode = PLAY_MODE_NORMAL;

                    modStop();
                    clearAll();

                    editor.currentMode = MODE_IDLE;
                    pointerSetMode(POINTER_MODE_IDLE, true);
                    setStatusMessage("ALL RIGHT", true);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_CLEARCANCEL:
                {
                    editor.ui.clearScreenShown = false;

                    setPrevStatusMessage();
                    pointerSetPreviousMode();

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SAMPLEU:
                {
                    sampleUpButton();
                    setupSampler(modEntry);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SAMPLED:
                {
                    sampleDownButton();
                    setupSampler(modEntry);

                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_FTUNEU:
                {
                    sampleFineTuneUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_FTUNED:
                {
                    sampleFineTuneDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SVOLUMEU:
                {
                    sampleVolumeUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SVOLUMED:
                {
                    sampleVolumeDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SLENGTHU:
                {
                    sampleLengthUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SLENGTHD:
                {
                    sampleLengthDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SREPEATU:
                {
                    sampleRepeatUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SREPEATD:
                {
                    sampleRepeatDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SREPLENU:
                {
                    sampleRepeatLengthUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_SREPLEND:
                {
                    sampleRepeatLengthDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_TEMPOU:
                {
                    tempoUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_TEMPOD:
                {
                    tempoDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_PLENGTHU:
                {
                    patternLengthUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_PLENGTHD:
                {
                    patternLengthDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_PATTERNU:
                {
                    patternUpButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_PATTERND:
                {
                    patternDownButton(modEntry);
                    input.mouse.lastClickedGUIButton = GUIButton;
                }
                break;

                case PTB_POSU:
                {
                    if (editor.playMode == PLAY_MODE_NORMAL)
                    {
                        positionUpButton(modEntry);
                        input.mouse.lastClickedGUIButton = GUIButton;
                    }
                }

                break;

                case PTB_POSD:
                {
                    if (editor.playMode == PLAY_MODE_NORMAL)
                    {
                        positionDownButton(modEntry);
                        input.mouse.lastClickedGUIButton = GUIButton;
                    }
                }
                break;

                default:
                {
                    input.mouse.buttonErrorMsgActive = true;
                    input.mouse.errorMsgCounter = 0;

                    setStatusMessage("NOT IMPLEMENTED !", false);
                    pointerErrorMode();
                }
                break;
            }
        }
    }
}

void handleMouseButtons(int lastMouseButton, int mouseX, int mouseY, MODULE *modEntry)
{
    if (input.mouse.leftButtonPressed == true)
    {
        int GUIButton = checkGUIButtons(mouseX, mouseY);
        if (GUIButton == -1)
        {
            return;
        }

        lastMouseButton = GUIButton;

        input.mouse.repeatCounter += 2;
        input.mouse.repeatCounter &= 0x000000FF;

        switch (lastMouseButton)
        {
            case PTB_SAMPLEU:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;

                    setupSampler(modEntry);
                    sampleUpButton();
                }
            }
            break;

            case PTB_SAMPLED:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;

                    setupSampler(modEntry);
                    sampleDownButton();
                }
            }
            break;

            case PTB_FTUNEU:
            {
                if (input.mouse.repeatCounter >= 3)
                {
                    input.mouse.repeatCounter = 0;
                    sampleFineTuneUpButton(modEntry);
                }
            }
            break;

            case PTB_FTUNED:
            {
                if (input.mouse.repeatCounter >= 3)
                {
                    input.mouse.repeatCounter = 0;
                    sampleFineTuneDownButton(modEntry);
                }
            }
            break;

            case PTB_SVOLUMEU:
            {
                if (input.mouse.repeatCounter >= 3)
                {
                    input.mouse.repeatCounter = 0;
                    sampleVolumeUpButton(modEntry);
                }
            }
            break;

            case PTB_SVOLUMED:
            {
                if (input.mouse.repeatCounter >= 3)
                {
                    input.mouse.repeatCounter = 0;
                    sampleVolumeDownButton(modEntry);
                }
            }
            break;

            case PTB_SLENGTHU:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    sampleLengthUpButton(modEntry);
                }
            }
            break;

            case PTB_SLENGTHD:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    sampleLengthDownButton(modEntry);
                }
            }
            break;

            case PTB_SREPEATU:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    sampleRepeatUpButton(modEntry);
                }
            }
            break;

            case PTB_SREPEATD:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    sampleRepeatDownButton(modEntry);
                }
            }
            break;

            case PTB_SREPLENU:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    sampleRepeatLengthUpButton(modEntry);
                }
            }
            break;

            case PTB_SREPLEND:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    sampleRepeatLengthDownButton(modEntry);
                }
            }
            break;

            case PTB_TEMPOU:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    tempoUpButton(modEntry);
                }
            }
            break;

            case PTB_TEMPOD:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;
                    tempoDownButton(modEntry);
                }
            }
            break;

            case PTB_PLENGTHU:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;
                    patternLengthUpButton(modEntry);
                }
            }
            break;

            case PTB_PLENGTHD:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;
                    patternLengthDownButton(modEntry);
                }
            }
            break;

            case PTB_PATTERNU:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;
                    patternUpButton(modEntry);
                }
            }
            break;

            case PTB_PATTERND:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;
                    patternDownButton(modEntry);
                }
            }
            break;

            case PTB_POSU:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;
                    positionUpButton(modEntry);
                }
            }
            break;

            case PTB_POSD:
            {
                if (input.mouse.repeatCounter >= 6)
                {
                    input.mouse.repeatCounter = 0;
                    positionDownButton(modEntry);
                }
            }
            break;

            case PTB_DO_SCROLLUP:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;

                    if (editor.diskop.scrollOffset > 0)
                    {
                        editor.diskop.scrollOffset--;
                    }

                    editor.diskop.forceUpdate = true;
                }
            }
            break;

            case PTB_DO_SCROLLDOWN:
            {
                if (input.mouse.repeatCounter >= 1)
                {
                    input.mouse.repeatCounter = 0;

                    editor.diskop.scrollOffset++;
                    editor.diskop.forceUpdate = true;
                }
            }
            break;

            default:
            break;
        }
    }
    else
    {
        input.mouse.repeatCounter = 0;
    }
}

void sampleUpButton(void)
{
    signed int val = editor.currentSample;

    if (input.mouse.rightButtonPressed == true)
    {
        val += 10;
    }
    else
    {
        val++;
    }

    if (val > 30)
    {
        val = 30;
    }

    editor.currentSample = val;
}


void sampleDownButton(void)
{
    signed int val = editor.currentSample;

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 10;
    }
    else
    {
        val--;
    }

    if (val < 0)
    {
        val = 0;
    }

    editor.currentSample = val;
}

void sampleFineTuneUpButton(MODULE *modEntry)
{
    signed int finetune = modEntry->samples[editor.currentSample].fineTune & 0x0F;
    if (finetune != 7)
    {
        modEntry->samples[editor.currentSample].fineTune++;
        modEntry->samples[editor.currentSample].fineTune = modEntry->samples[editor.currentSample].fineTune & 0x0F;
    }
}

void sampleFineTuneDownButton(MODULE *modEntry)
{
    signed int finetune = modEntry->samples[editor.currentSample].fineTune & 0x0F;
    if (finetune != 8)
    {
        modEntry->samples[editor.currentSample].fineTune--;
        modEntry->samples[editor.currentSample].fineTune = modEntry->samples[editor.currentSample].fineTune & 0x0F;
    }
}

void sampleVolumeUpButton(MODULE *modEntry)
{
    signed int val = modEntry->samples[editor.currentSample].volume;

    if (input.mouse.rightButtonPressed == true)
    {
        val += 10;
    }
    else
    {
        val++;
    }

    if (val > 64)
    {
        val = 64;
    }

    modEntry->samples[editor.currentSample].volume = (unsigned char)val;
}

void sampleVolumeDownButton(MODULE *modEntry)
{
    signed int val = modEntry->samples[editor.currentSample].volume;

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 10;
    }
    else
    {
        val--;
    }

    if (val < 0)
    {
        val = 0;
    }

    modEntry->samples[editor.currentSample].volume = (unsigned char)val;
}

void sampleLengthUpButton(MODULE *modEntry)
{
    signed int val = modEntry->samples[editor.currentSample].length;

    if (input.mouse.rightButtonPressed == true)
    {
        val += 16;
    }
    else
    {
        val += 2;
    }

    if (val > 0x1FFFF)
    {
        val = 0x1FFFF;
    }

    modEntry->samples[editor.currentSample].length = val;

    if (modEntry->samples[editor.currentSample].loopStart > (val - 2))
    {
        modEntry->samples[editor.currentSample].loopStart = val - 2;
    }

    if ((modEntry->samples[editor.currentSample].loopLength > val) && (val > 2))
    {
        modEntry->samples[editor.currentSample].loopLength = val;
    }
}

void sampleLengthDownButton(MODULE *modEntry)
{
    signed int val = modEntry->samples[editor.currentSample].length;

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 16;
    }
    else
    {
        val -= 2;
    }

    if (val < 0)
    {
        val = 0;
    }

    modEntry->samples[editor.currentSample].length = val;

    if (val == 0)
    {
        modEntry->samples[editor.currentSample].loopStart = 0;
        modEntry->samples[editor.currentSample].loopLength = 2;
    }
    else
    {
        if (modEntry->samples[editor.currentSample].loopStart > (val - 2))
        {
            modEntry->samples[editor.currentSample].loopStart = val - 2;
        }

        if ((modEntry->samples[editor.currentSample].loopLength > val) && (val > 2))
        {
            modEntry->samples[editor.currentSample].loopLength = val;
        }
    }
}

void sampleRepeatUpButton(MODULE *modEntry)
{
    int val = modEntry->samples[editor.currentSample].loopStart;
    int loop_len = modEntry->samples[editor.currentSample].loopLength;
    int len = modEntry->samples[editor.currentSample].length;

    if (len == 0)
    {
        modEntry->samples[editor.currentSample].loopStart = 0;

        return;
    }

    if (input.mouse.rightButtonPressed == true)
    {
        val += 16;
    }
    else
    {
        val += 2;
    }

    if (val > (len - loop_len))
    {
        val = len - loop_len;
    }

    modEntry->samples[editor.currentSample].loopStart = val;
}

void sampleRepeatDownButton(MODULE *modEntry)
{
    signed int val = modEntry->samples[editor.currentSample].loopStart;
    int len = modEntry->samples[editor.currentSample].length;

    if (len == 0)
    {
        modEntry->samples[editor.currentSample].loopStart = 0;

        return;
    }

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 16;
    }
    else
    {
        val -= 2;
    }

    if (val < 0)
    {
        val = 0;
    }

    modEntry->samples[editor.currentSample].loopStart = val;
}

void sampleRepeatLengthUpButton(MODULE *modEntry)
{
    int val = modEntry->samples[editor.currentSample].loopLength;
    int loopStart = modEntry->samples[editor.currentSample].loopStart;
    int len = modEntry->samples[editor.currentSample].length;

    if (len == 0)
    {
        modEntry->samples[editor.currentSample].loopLength = 2;

        return;
    }

    if (input.mouse.rightButtonPressed == true)
    {
        val += 16;
    }
    else
    {
        val += 2;
    }

    if (val > (len - loopStart))
    {
        val = len - loopStart;
    }

    modEntry->samples[editor.currentSample].loopLength = val;
}

void sampleRepeatLengthDownButton(MODULE *modEntry)
{
    signed int val = modEntry->samples[editor.currentSample].loopLength;
    int len = modEntry->samples[editor.currentSample].length;

    if (len == 0)
    {
        modEntry->samples[editor.currentSample].loopLength = 2;

        return;
    }

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 16;
    }
    else
    {
        val -= 2;
    }

    if (val < 2)
    {
        val = 2;
    }

    modEntry->samples[editor.currentSample].loopLength = val;
}

void tempoUpButton(MODULE *modEntry)
{
    signed int val = modEntry->currBPM;

    if (input.mouse.rightButtonPressed == true)
    {
        val += 10;
    }
    else
    {
        val++;
    }

    if (val > 255)
    {
        val = 255;
    }

    modEntry->currBPM = (unsigned char)val;
    modSetTempo(modEntry->currBPM);
}

void tempoDownButton(MODULE *modEntry)
{
    signed int val = modEntry->currBPM;

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 10;
    }
    else
    {
        val--;
    }

    if (val < 32)
    {
        val = 32;
    }

    modEntry->currBPM = (unsigned char)val;
    modSetTempo(modEntry->currBPM);
}

void patternLengthUpButton(MODULE *modEntry)
{
    signed int val = modEntry->head.orderCount;

    if (input.mouse.rightButtonPressed == true)
    {
        val += 10;
    }
    else
    {
        val++;
    }

    if (val > 128)
    {
        val = 128;
    }

    modEntry->head.orderCount = (unsigned char)val;
}

void patternLengthDownButton(MODULE *modEntry)
{
    signed int val = modEntry->head.orderCount;

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 10;
    }
    else
    {
        val--;
    }

    if (val < 1)
    {
        val = 1;
    }

    if (modEntry->currOrder > (val - 1))
    {
        modEntry->currOrder = val - 1;
    }

    modEntry->head.orderCount = (unsigned char)val;
}

void patternUpButton(MODULE *modEntry)
{
    signed int val = modEntry->head.order[modEntry->currOrder];

    if (input.mouse.rightButtonPressed == true)
    {
        val += 10;
    }
    else
    {
        val++;
    }

    if (val > (modEntry->MAX_PATTERNS - 1))
    {
        val = modEntry->MAX_PATTERNS - 1;
    }

    modEntry->head.order[modEntry->currOrder] = modEntry->currPattern = (unsigned char)val;

    calcRealModSize(modEntry);
}

void patternDownButton(MODULE *modEntry)
{
    signed int val = modEntry->head.order[modEntry->currOrder];

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 10;
    }
    else
    {
        val--;
    }

    if (val < 0)
    {
        val = 0;
    }

    modEntry->head.order[modEntry->currOrder] = modEntry->currPattern = (unsigned char)val;

    calcRealModSize(modEntry);
}

void positionUpButton(MODULE *modEntry)
{
    signed int val = modEntry->currOrder;

    if (input.mouse.rightButtonPressed == true)
    {
        val += 10;
    }
    else
    {
        val++;
    }

    if (val > (modEntry->head.orderCount - 1))
    {
        val = modEntry->head.orderCount - 1;
    }

    if (modEntry->currOrder != (modEntry->head.orderCount - 1))
    {
        modSetPos(val, -1);
    }
}

void positionDownButton(MODULE *modEntry)
{
    signed int val = modEntry->currOrder;

    if (input.mouse.rightButtonPressed == true)
    {
        val -= 10;
    }
    else
    {
        val--;
    }

    if (val < 0)
    {
        val = 0;
    }

    if (modEntry->currOrder != 0)
    {
        modSetPos(val, -1);
    }
}

void diskOpLoadFile(const SDL_Surface *screen, int fileEntryRow, MODULE *modEntry)
{
    char *dirPath, *filePath;

    if (diskOpEntryIsDir(fileEntryRow))
    {
#ifdef WIN32
        dirPath = diskOpGetEntryPath(fileEntryRow);
        if (dirPath != NULL)
        {
            editor.diskop.forceUpdate = true;
            editor.diskop.scrollOffset = 0;

            diskOpSetPath(dirPath);
        }
#else
        char *setPath;

        setPath = diskOpGetEntry(fileEntryRow);
        dirPath = diskOpGetEntryPath(fileEntryRow);
        if (dirPath != NULL)
        {
            editor.diskop.forceUpdate = true;
            editor.diskop.scrollOffset = 0;

            chdir(setPath);
            diskOpSetPath(dirPath);
        }
#endif
    }
    else
    {
        filePath = diskOpGetEntryPath(fileEntryRow);
        if (filePath != NULL)
        {
            MODULE *newMod = modLoad(filePath);
            if (newMod != NULL)
            {
                editor.ui.diskOpScreenShown = false;

                modStop();

                modEntry->moduleLoaded = false;
                modFree(modEntry);

                modEntry = newMod;
                setPlayerModSource(modEntry);
                setMainModSource(modEntry);

                modSetSpeed(6);
                modSetTempo(125);
                modSetPos(0, 0);

                modEntry->moduleLoaded = true;

                editor.currentMode = MODE_IDLE;
                pointerSetMode(POINTER_MODE_IDLE, true);
                setStatusMessage("ALL RIGHT", true);
            }
            else
            {
                input.mouse.buttonErrorMsgActive = true;
                input.mouse.errorMsgCounter = 0;

                setStatusMessage("MOD LOAD ERROR !", false);
                pointerErrorMode();
            }
        }
    }
}
