#include "pt_header.h"
#include "pt_textout.h"

static const char noteNameTable[36][4] =
{
    "C-1", "C#1", "D-1", "D#1", "E-1", "F-1", "F#1" ,"G-1", "G#1", "A-1", "A#1", "B-1",
    "C-2", "C#2", "D-2", "D#2", "E-2", "F-2", "F#2" ,"G-2", "G#2", "A-2", "A#2", "B-2",
    "C-3", "C#3", "D-3", "D#3", "E-3", "F-3", "F#3" ,"G-3", "G#3", "A-3", "A#3", "B-3"
};

static const short periodTable[36] =
{
    856, 808, 762, 720, 678, 640, 604, 570, 538, 508, 480, 453,
    428, 404, 381, 360, 339, 320, 302, 285, 269, 254, 240, 226,
    214, 202, 190, 180, 170, 160, 151, 143, 135, 127, 120, 113
};

inline static int periodToNote(int period)
{
    int i;

    if (period == 0)
    {
        return -1; // No period
    }

    if ((period < 113) || (period > 856))
    {
        return 36; // Out of range
    }

    // OK, let's fetch the note
    for (i = 0; i < 36; ++i)
    {
        if (periodTable[i] <= period)
        {
            break;
        }
    }

    return i;
}

void drawPattern(unsigned int *frameBuffer, const MODULE *modEntry)
{
    int i, j;
    int putXOffset, putYOffset;
    int rowMiddlePos, rowDispCheck, rowData;
    int tempNote;

    for (i = 0; i < 15; ++i)
    {
        rowMiddlePos = i - 7;
        rowDispCheck = modEntry->currRow + rowMiddlePos;

        if ((rowDispCheck >= 0) && (rowDispCheck < MOD_ROWS))
        {
            rowData = rowDispCheck * 4;
            putYOffset = 140 + (i * 7);

            if (i == 7) // Are we on the play row (middle)?
            {
                putYOffset++; // align font to play row plane

                // Put current row number
                printTwoDecimalsBig(frameBuffer, rowMiddlePos + modEntry->currRow, 8, putYOffset, 0x00000000);

                // Pattern data
                for (j = 0; j < PAULA_CHANNELS; ++j)
                {
                    modnote_t note = modEntry->patterns[modEntry->currPattern][rowData + j];

                    putXOffset = 26 + (j * 72);

                    tempNote = periodToNote(note.period);
                    if (tempNote == -1)
                    {
                        // No note
                        charOutBig(frameBuffer, putXOffset +  6, putYOffset, '-', 0x00000000);
                        charOutBig(frameBuffer, putXOffset + 14, putYOffset, '-', 0x00000000);
                        charOutBig(frameBuffer, putXOffset + 22, putYOffset, '-', 0x00000000);
                    }
                    else if (tempNote == 36)
                    {
                        // Out of range, invalid note
                        charOutBig(frameBuffer, putXOffset +  6, putYOffset, '?', 0x00000000);
                        charOutBig(frameBuffer, putXOffset + 14, putYOffset, '?', 0x00000000);
                        charOutBig(frameBuffer, putXOffset + 22, putYOffset, '?', 0x00000000);
                    }
                    else
                    {
                        charOutBig(frameBuffer, putXOffset +  6, putYOffset, noteNameTable[tempNote][0], 0x00000000);
                        charOutBig(frameBuffer, putXOffset + 14, putYOffset, noteNameTable[tempNote][1], 0x00000000);
                        charOutBig(frameBuffer, putXOffset + 22, putYOffset, noteNameTable[tempNote][2], 0x00000000);
                    }

                    printTwoHexBig(frameBuffer, note.sample,  putXOffset + 30, putYOffset, 0x00000000);
                    printOneHexBig(frameBuffer, note.command, putXOffset + 46, putYOffset, 0x00000000);
                    printTwoHexBig(frameBuffer, note.param,   putXOffset + 54, putYOffset, 0x00000000);
                }
            }
            else
            {
                if (i > 7)
                {
                    // Beyond play row, jump some pixels out of the plane
                    putYOffset += 7;
                }

                // Put current row number
                printTwoDecimals(frameBuffer, rowMiddlePos + modEntry->currRow, 8, putYOffset, 0x003344FF);

                // Pattern data
                for (j = 0; j < PAULA_CHANNELS; ++j)
                {
                    modnote_t note = modEntry->patterns[modEntry->currPattern][rowData + j];

                    putXOffset = 26 + (j * 72);

                    tempNote = periodToNote(note.period);
                    if (tempNote == -1)
                    {
                        // No note
                        charOut(frameBuffer, putXOffset +  6, putYOffset, '-', 0x003344FF);
                        charOut(frameBuffer, putXOffset + 14, putYOffset, '-', 0x003344FF);
                        charOut(frameBuffer, putXOffset + 22, putYOffset, '-', 0x003344FF);
                    }
                    else if (tempNote == 36)
                    {
                        // Out of range, invalid note
                        charOut(frameBuffer, putXOffset +  6, putYOffset, '?', 0x003344FF);
                        charOut(frameBuffer, putXOffset + 14, putYOffset, '?', 0x003344FF);
                        charOut(frameBuffer, putXOffset + 22, putYOffset, '?', 0x003344FF);
                    }
                    else
                    {
                        charOut(frameBuffer, putXOffset +  6, putYOffset, noteNameTable[tempNote][0], 0x003344FF);
                        charOut(frameBuffer, putXOffset + 14, putYOffset, noteNameTable[tempNote][1], 0x003344FF);
                        charOut(frameBuffer, putXOffset + 22, putYOffset, noteNameTable[tempNote][2], 0x003344FF);
                    }

                    printTwoHex(frameBuffer, note.sample,  putXOffset + 30, putYOffset, 0x003344FF);
                    printOneHex(frameBuffer, note.command, putXOffset + 46, putYOffset, 0x003344FF);
                    printTwoHex(frameBuffer, note.param,   putXOffset + 54, putYOffset, 0x003344FF);
                }
            }
        }
    }
}
