// Sampler codes taken from the ptplay project source and cleaned up

#include <SDL/SDL.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "pt_header.h"
#include "pt_palette.h"
#include "pt_textout.h"
#include "pt_audio.h"
#include "pt_palette.h"
#include "pt_gfx_loop_pins.h"

#ifdef _MSC_VER
inline static int ftol(const float f)
{
    __asm
    {
        fld f
        push eax
        fistp dword ptr [esp]
        pop eax
    }
}
#else
#define ftol(f) (signed int)(f)
#endif

static int rangeStart, rangeEnd;
static int displayStart, displayEnd;
static float scaleFactor;
static int sampleOffset;

inline void drawLine(unsigned int *frameBuffer, int w, int x1, int y1, int x2, int y2, unsigned int color);

unsigned int samplerBuffer[314 * 64];

void setupSampler(const MODULE *modEntry)
{
    rangeStart = 0;
    rangeEnd = 0;
    displayStart = 0;
    displayEnd = modEntry->samples[editor.currentSample].length;
    scaleFactor = (float)(displayEnd - displayStart) / 314.0f;
    sampleOffset = displayStart;
    editor.ui.sampleDataDrawn = false;
}

void drawSampler(unsigned int *frameBuffer, const MODULE_SAMPLE *s, const MODULE *modEntry)
{
    int i, x, y, h, h2, l, k;
    int step1, step2;
    int x1, x2;
    int srcY, dstY, dstX1, dstX2;
    int pixelOut;
    float res;

    float fScaleResTmp = 314.0f / (float)(displayEnd - displayStart);

    // DRAW SAMPLE DATA ONCE
    if (editor.ui.sampleDataDrawn == false)
    {
        memset(samplerBuffer, 0, 314 * 64 * 4);

        for (i = 0; i < 313; ++i)
        {
            step1 = ftol(scaleFactor * (float)i);
            step2 = ftol(scaleFactor * (float)(i + 1));

            if (s->length <= 2)
            {
                h = 0;
                h2 = 0;
            }
            else
            {
                h = modEntry->sampleData[s->offset + sampleOffset + step1] / 4;
                h2 = modEntry->sampleData[s->offset + sampleOffset + step2] / 4;
            }

            drawLine(samplerBuffer, 314, i, 31 - h, i + 1, 31 - h2, 0x00FFDD00);
        }

        editor.ui.sampleDataDrawn = true;
    }

    // Copy waveform buffer to screen
    for (y = 0; y < 64; ++y)
    {
        for (x = 0; x < 314; ++x)
        {
            frameBuffer[(138 + y) * SCREEN_W + (3 + x)] = samplerBuffer[y * 314 + x];
        }
    }

    printSixDecimals(frameBuffer, displayEnd - displayStart, 265, 214, 0x00000000);

    if (s->loopLength > 2)
    {
        textOut(frameBuffer, 289, 225, "ON", 0x00000000);
    }
    else
    {
        textOut(frameBuffer, 289, 225, "OFF", 0x00000000);
    }

    textOut(frameBuffer, 289, 236, "C-3", 0x00000000);
    textOut(frameBuffer, 289, 247, "C-3", 0x00000000);

    // DRAW LOOP POINT PINS
    x1 = s->loopStart;
    x2 = x1 + s->loopLength;

    if (x2 > 2)
    {
        res = fScaleResTmp;

        x1 -= sampleOffset;
        x2 -= sampleOffset;
        x1 = ftol((float)x1 * res);
        x2 = ftol((float)x2 * res);

        for (x = 0; x < 4; ++x)
        {
            // Left and right loop pin
            dstX1 = x + x1;
            dstX2 = x + x2 + 3;

            for (y = 0; y < 64; ++y)
            {
                srcY = (y * 4) + x;
                dstY = (138 + y) * SCREEN_W;

                // Left
                pixelOut = loopPinLeftBitmap[srcY];
                if (pixelOut != COLOR_KEY)
                {
                    frameBuffer[dstY + dstX1] = pixelOut;
                }

                // Right
                if (dstX2 < SCREEN_W)
                {
                    pixelOut = loopPinRightBitmap[srcY];
                    if (pixelOut != COLOR_KEY)
                    {
                        frameBuffer[dstY + dstX2] = pixelOut;
                    }
                }
            }
        }
    }

    // DRAW SAMPLE POSITION LINE
    res = fScaleResTmp;
    k = 0;

    for (i = 0; i < 4; ++i)
    {
        k = getChReadPos(i);
        l = -1;

        if (modEntry->channels[i].sample != 0)
        {
            if (modEntry->channels[i].sample == (editor.currentSample + 1))
            {
                if ((k >= 0) && (k < s->length))
                {
                    l = k;
                    if (s->loopLength > 2)
                    {
                        if (l >= (s->loopStart + s->loopLength))
                        {
                            l -= s->loopLength;
                        }
                    }
                    else
                    {
                        if (l >= s->length)
                        {
                            l = -1;
                        }
                    }
                }
            }
        }

        if (l != -1)
        {
            h = l;
            h -= sampleOffset;
            h = ftol((float)h * res);

            if (h < 316)
            {
                drawLine(frameBuffer, SCREEN_W, 3 + h, 169 - 31, 3 + h, 169 + 32, 0x0000FFFF);
            }
        }
    }
}

void drawLine(unsigned int *frameBuffer, int w, int x1, int y1, int x2, int y2, unsigned int color)
{
    int d, x, y, ax, ay, sx, sy, dx, dy;

    dx = x2 - x1;
    ax = ABS(dx) * 2;
    sx = SGN(dx);

    dy = y2 - y1;
    ay = ABS(dy) * 2;
    sy = SGN(dy);

    x = x1;
    y = y1;

    if (ax > ay)
    {
        d = ay - (ax / 2);

        while (true)
        {
            frameBuffer[y * w + x] = color;

            if (x == x2)
            {
                return;
            }

            if (d >= 0)
            {
                y += sy;
                d -= ax;
            }

            x += sx;
            d += ay;
        }
    }
    else
    {
        d = ax - (ay / 2);

        while (true)
        {
            frameBuffer[y * w + x] = color;

            if (y == y2)
            {
                return;
            }

            if (d >= 0)
            {
                x += sx;
                d -= ay;
            }

            y += sy;
            d += ax;
        }
    }
}
