#include "pt_header.h"
#include "pt_helpers.h"
#include "pt_gfx_font.h"

static const char hexTable[16] =
{
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
};

void charOut(unsigned int *frameBuffer, int putX, int putY, char ch, unsigned int fontColor)
{
    int i;

    unsigned int *frameBufferPointer = &frameBuffer[putY * SCREEN_W + putX];
    char *fontPointer = (char *)&fontBitmap[ch * FONT_CHAR_W * FONT_CHAR_H];

    for (i = 0; i < (FONT_CHAR_W * FONT_CHAR_H); ++i)
    {
        if (*fontPointer++ == 1)
        {
            *frameBufferPointer = fontColor;
        }

        if ((i & (FONT_CHAR_W - 1)) == (FONT_CHAR_W - 1))
        {
            frameBufferPointer += (SCREEN_W - (FONT_CHAR_W - 1));
        }
        else
        {
            frameBufferPointer++;
        }
    }
}

void charOutBig(unsigned int *frameBuffer, int putX, int putY, char ch, unsigned int fontColor)
{
    int i;

    unsigned int *frameBufferPointer = &frameBuffer[putY * SCREEN_W + putX];
    char *fontPointer = (char *)&fontBitmap[ch * FONT_CHAR_W * FONT_CHAR_H];

    for (i = 0; i < (FONT_CHAR_W * FONT_CHAR_H * 2); ++i)
    {
        if (*fontPointer++ == 1)
        {
            *frameBufferPointer = fontColor;
        }

        if ((i & (FONT_CHAR_W - 1)) == (FONT_CHAR_W - 1))
        {
            if ((i & ((FONT_CHAR_W * 2) - 1)) != ((FONT_CHAR_W * 2) - 1))
            {
                fontPointer -= FONT_CHAR_W;
            }

            frameBufferPointer += (SCREEN_W - (FONT_CHAR_W - 1));
        }
        else
        {
            frameBufferPointer++;
        }
    }
}

void textOut(unsigned int *frameBuffer, int putX, int putY, const char *text, unsigned int fontColor)
{
    int i, j;

    for (i = 0; text[i] != 0; ++i)
    {
        unsigned int *frameBufferPointer = &frameBuffer[putY * SCREEN_W + (putX + (i * FONT_CHAR_W))];
        char *fontPointer = (char *)&fontBitmap[text[i] * FONT_CHAR_W * FONT_CHAR_H];

        for (j = 0; j < (FONT_CHAR_W * FONT_CHAR_H); ++j)
        {
            if (*fontPointer++ == 1)
            {
                *frameBufferPointer = fontColor;
            }

            if ((j & (FONT_CHAR_W - 1)) == (FONT_CHAR_W - 1))
            {
                frameBufferPointer += (SCREEN_W - (FONT_CHAR_W - 1));
            }
            else
            {
                frameBufferPointer++;
            }
        }
    }
}

void textOutBig(unsigned int *frameBuffer, int putX, int putY, const char *text, unsigned int fontColor)
{
    int i, j;

    for (i = 0; text[i] != 0; ++i)
    {
        unsigned int *frameBufferPointer = &frameBuffer[putY * SCREEN_W + (putX + (i * FONT_CHAR_W))];
        char *fontPointer = (char *)&fontBitmap[text[i] * FONT_CHAR_W * FONT_CHAR_H];

        for (j = 0; j < (FONT_CHAR_W * FONT_CHAR_H * 2); ++j)
        {
            if (*fontPointer++ == 1)
            {
                *frameBufferPointer = fontColor;
            }

            if ((j & (FONT_CHAR_W - 1)) == (FONT_CHAR_W - 1))
            {
                if ((j & ((FONT_CHAR_W * 2) - 1)) != ((FONT_CHAR_W * 2) - 1))
                {
                    fontPointer -= FONT_CHAR_W;
                }

                frameBufferPointer += (SCREEN_W - (FONT_CHAR_W - 1));
            }
            else
            {
                frameBufferPointer++;
            }
        }
    }
}

void printTwoDecimals(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x + 8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x    , y, '0' + (value % 10), fontColor);
}

void printTwoDecimalsBig(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOutBig(frameBuffer, x +  8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOutBig(frameBuffer, x     , y, '0' + (value % 10), fontColor);
}

void printThreeDecimals(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x + 16, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x +  8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x     , y, '0' + (value % 10), fontColor);
}

void printFourDecimals(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x + 24, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 16, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x +  8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x     , y, '0' + (value % 10), fontColor);
}

void printFiveDecimals(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x + 32, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 24, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 16, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x +  8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x     , y, '0' + (value % 10), fontColor);
}

void printSixDecimals(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x + 40, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 32, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 24, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 16, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x +  8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x     , y, '0' + (value % 10), fontColor);
}

void printSevenDecimals(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x + 48, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 40, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 32, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 24, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x + 16, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x +  8, y, '0' + (value % 10), fontColor);
    value /= 10;
    charOut(frameBuffer, x     , y, '0' + (value % 10), fontColor);
}

void printOneHex(unsigned int *frameBuffer, unsigned char value, int x, int y, unsigned int fontColor)
{
    // *don't* pass > 0x0F to this!
    charOut(frameBuffer, x, y, hexTable[value], fontColor);
}

void printOneHexBig(unsigned int *frameBuffer, unsigned char value, int x, int y, unsigned int fontColor)
{
    // *don't* pass > 0x0F to this!
    charOutBig(frameBuffer, x, y, hexTable[value], fontColor);
}

void printTwoHex(unsigned int *frameBuffer, unsigned char value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x     , y, hexTable[HI_NYBBLE(value)], fontColor);
    charOut(frameBuffer, x +  8, y, hexTable[LO_NYBBLE(value)], fontColor);
}

void printTwoHexBig(unsigned int *frameBuffer, unsigned char value, int x, int y, unsigned int fontColor)
{
    charOutBig(frameBuffer, x     , y, hexTable[HI_NYBBLE(value)], fontColor);
    charOutBig(frameBuffer, x +  8, y, hexTable[LO_NYBBLE(value)], fontColor);
}

void printFourHex(unsigned int *frameBuffer, unsigned short value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x     , y, hexTable[(value                  ) >> (3 * 4)], fontColor);
    charOut(frameBuffer, x +  8, y, hexTable[(value & (15 << (2 * 4))) >> (2 * 4)], fontColor);
    charOut(frameBuffer, x + 16, y, hexTable[(value & (15 << (1 * 4))) >> (1 * 4)], fontColor);
    charOut(frameBuffer, x + 24, y, hexTable[(value & (15 << (0 * 4))) >> (0 * 4)], fontColor);
}

void printFiveHex(unsigned int *frameBuffer, unsigned int value, int x, int y, unsigned int fontColor)
{
    charOut(frameBuffer, x     , y, hexTable[(value                  ) >> (4 * 4)], fontColor);
    charOut(frameBuffer, x +  8, y, hexTable[(value & (15 << (3 * 4))) >> (3 * 4)], fontColor);
    charOut(frameBuffer, x + 16, y, hexTable[(value & (15 << (2 * 4))) >> (2 * 4)], fontColor);
    charOut(frameBuffer, x + 24, y, hexTable[(value & (15 << (1 * 4))) >> (1 * 4)], fontColor);
    charOut(frameBuffer, x + 32, y, hexTable[(value & (15 << (0 * 4))) >> (0 * 4)], fontColor);
}

void scrollerTextOut(unsigned int *frameBuffer, int putX, int putY, int bufWidth, const char *text, unsigned int fontColor)
{
    int i;
    int x, y;
    int src_tmp, dst_tmp;
    int src, dst;

    for (i = 0; text[i] != 0; ++i)
    {
        src_tmp = FONT_CHAR_H * text[i];
        dst_tmp = (i * (FONT_CHAR_W - 1)) + putX;

        for (y = 0; y < FONT_CHAR_H; ++y)
        {
            src = (src_tmp + y) * FONT_CHAR_W;
            dst = ((putY + y) * bufWidth) + dst_tmp;

            for (x = 0; x < FONT_CHAR_W; ++x)
            {
                if (fontBitmap[src++] == 1)
                {
                    frameBuffer[dst + x] = fontColor;
                }
            }
        }
    }
}
